@@ -17,10 +17,8 @@
  *  2010 Ilya Popov <ilia_popov@rambler.ru>
  *  2012 Emil Martinec <ejmartin@uchicago.edu>
  *  2014 Ingo Weyrich <heckflosse@i-weyrich.de>
- */
-
-#ifndef CPLX_WAVELET_LEVEL_H_INCLUDED
-#define CPLX_WAVELET_LEVEL_H_INCLUDED
+*/
+#pragma once
 
 #include <cstddef>
 #include "rt_math.h"
@@ -274,7 +272,7 @@ template<typename T> void wavelet_level<T>::SynthesisFilterHaarVertical (const T
         #pragma omp for nowait
 #endif
 
-        for(int i = 0; i < skip; i++)
+        for(int i = 0; i < std::min(skip, height); i++)
         {
             for(int j = 0; j < width; j++) {
                 dst[width * i + j] = (srcLo[i * width + j] + srcHi[i * width + j]);
@@ -759,5 +757,3 @@ template<typename T> template<typename E> void wavelet_level<T>::reconstruct_lev
 }
 #endif
 }
-
-#endif
