@@ -19,9 +19,13 @@
 
 #pragma once
 
+#include <cstdint>
+#include <iostream>
+
 #include "myfile.h"
 #include <csetjmp>
-
+#include "dnggainmap.h"
+#include "settings.h"
 
 class DCraw
 {
@@ -29,7 +33,7 @@ public:
 	typedef unsigned short ushort;
 	typedef unsigned char uchar;
 	typedef unsigned short (*dcrawImage_t)[4];
-#ifdef WIN32
+#ifdef _WIN32
 	typedef __int64 INT64;
 	typedef unsigned __int64 UINT64;
 #else
@@ -72,6 +76,20 @@ public:
     }
 
 protected:
+    enum class CropMode : std::uint_fast16_t {                                   // RT
+        NA = 0,                                                                  // RT
+        FullFrameOnGfx = 1,                                                      // RT
+        SportsFinderMode = 2,                                                    // RT
+        ElectronicShutter1_25xCrop = 4                                           // RT
+    };                                                                           // RT
+    // stores the cropdata read from the file                                       RT
+    struct CropData {                                                            // RT
+        std::uint_fast16_t  width,                                               // RT
+                            height,                                              // RT
+                            top_margin,                                          // RT
+                            left_margin;                                         // RT
+        CropMode crop_mode = CropMode::NA;                                       // RT
+    } read_crop;                                                                 // RT
     int exif_base, ciff_base, ciff_len;
     rtengine::IMFILE *ifp;
     FILE *ofp;
@@ -165,6 +183,8 @@ protected:
         PanasonicRW2Info(): bpp(0), encoding(0) {}
     };
     PanasonicRW2Info RT_pana_info;
+    std::vector<GainMap> gainMaps;
+
 public:
     struct CanonCR3Data {
         // contents of tag CMP1 for relevant track in CR3 file
@@ -193,6 +213,18 @@ public:
         int crx_track_selected;
         short CR3_CTMDtag;
     };
+
+    bool isBayer() const
+    {
+        return (filters != 0 && filters != 9);
+    }
+
+    const std::vector<GainMap>& getGainMaps() const {
+        return gainMaps;
+    }
+
+    bool isGainMapSupported() const;
+
     struct CanonLevelsData {
         unsigned cblack[4];
         unsigned white;
@@ -200,6 +232,7 @@ public:
         bool white_ok;
         CanonLevelsData(): cblack{0}, white{0}, black_ok(false), white_ok(false) {}
     };
+
 protected:
     CanonCR3Data RT_canon_CR3_data;
     
@@ -432,6 +465,7 @@ void parse_hasselblad_gain();
 void hasselblad_load_raw();
 void leaf_hdr_load_raw();
 void unpacked_load_raw();
+void unpacked_load_raw_FujiDBP();
 void sinar_4shot_load_raw();
 void imacon_full_load_raw();
 void packed_load_raw();
