@@ -90,6 +90,19 @@ constexpr double ACESp0_xyz[3][3] = {
     {0.00845768,  -0.01403193, 1.21893277}
 };
 
+constexpr double xyz_jdcmax[3][3] = {//prim red 0.734702 0.265302 gr 0.021908 0.930288 bl 0.120593 0.001583
+    {0.8394088, 0.0163780, 0.1084133},
+    {0.3031122, 0.6954651, 0.0014227},
+    {-0.000048, 0.0357376, 0.7891671}
+};
+
+constexpr double jdcmax_xyz[3][3] = {
+    {1.1984508, -0.0197646, -0.1646037},
+    {-0.5223824, 1.4466349, 0.0691553},
+    {0.0236634,  -0.0655113, 1.2640260}
+};
+
+
 constexpr double xyz_ACESp1[3][3] = {
     {0.689697, 0.149944, 0.124559},
     {0.284448, 0.671758  , 0.043794},
