@@ -17,7 +17,8 @@
  *  along with RawTherapee.  If not, see <https://www.gnu.org/licenses/>.
  */
 #include <fftw3.h>
-#include "../rtgui/profilestorecombobox.h"
+#include <glibmm/miscutils.h>
+#include <glibmm/ustring.h>
 #include "color.h"
 #include "rtengine.h"
 #include "iccstore.h"
@@ -33,6 +34,7 @@
 #include "profilestore.h"
 #include "../rtgui/threadutils.h"
 #include "rtlensfun.h"
+#include "metadata.h"
 #include "procparams.h"
 
 namespace rtengine
@@ -113,6 +115,8 @@ int init (const Settings* s, const Glib::ustring& baseDir, const Glib::ustring&
 }
 
     Color::init ();
+    Exiv2Metadata::init();
+
     delete lcmsMutex;
     lcmsMutex = new MyMutex;
     fftwMutex = new MyMutex;
@@ -121,6 +125,7 @@ int init (const Settings* s, const Glib::ustring& baseDir, const Glib::ustring&
 
 void cleanup ()
 {
+    Exiv2Metadata::cleanup();
     ProcParams::cleanup ();
     Color::cleanup ();
     RawImageSource::cleanup ();
