@@ -54,7 +54,6 @@ public:
     LCPModelCommon();
 
     bool empty() const;  // is it empty
-    void print() const;  // printf all values
     void merge(const LCPModelCommon& a, const LCPModelCommon& b, float facA);
     void prepareParams(
         int fullWidth,
@@ -106,7 +105,6 @@ public:
         LCPModelCommon *pCorr3
     ) const; // Interpolates between the persModels frames
 
-    void print() const;
 
 //private:
     // Common data
@@ -168,7 +166,7 @@ private:
 class LensCorrection {
 public:
     virtual ~LensCorrection() {}
-    virtual void correctDistortion(double &x, double &y, int cx, int cy, double scale) const = 0;
+    virtual void correctDistortion(double &x, double &y, int cx, int cy) const = 0;
     virtual bool isCACorrectionAvailable() const = 0;
     virtual void correctCA(double &x, double &y, int cx, int cy, int channel) const = 0;
     virtual void processVignette(int width, int height, float** rawData) const = 0;
@@ -196,7 +194,7 @@ public:
     );
 
 
-    void correctDistortion(double &x, double &y, int cx, int cy, double scale) const override;  // MUST be the first stage
+    void correctDistortion(double &x, double &y, int cx, int cy) const override;
     bool isCACorrectionAvailable() const override;
     void correctCA(double& x, double& y, int cx, int cy, int channel) const override;
     void processVignette(int width, int height, float** rawData) const override;
