@@ -137,7 +137,7 @@ enum ProcEventCode {
     EvHLComprThreshold = 107,
     EvResizeBoundingBox = 108,
     EvResizeAppliesTo = 109,
-    EvLAvoidColorShift = 110,
+    //EvLAvoidColorShift = 110,
     obsolete_111 = 111,    // obsolete
     EvLRSTProtection = 112,
     EvDemosaicDCBIter = 113,
@@ -617,7 +617,7 @@ enum ProcEventCode {
     Evlocallabadjblur = 587,
     Evlocallabbilateral = 588,
     Evlocallabsensiden = 589,
-    Evlocallabavoid = 590,
+   // Evlocallabavoid = 590,
     Evlocallabsharcontrast = 591,
     EvLocenacontrast = 592,
     Evlocallablcradius = 593,
@@ -1067,7 +1067,7 @@ enum ProcEventCode {
     Evlocallabnlgam = 1037,
     Evlocallabdivgr = 1038,
     EvLocallabSpotavoidrad = 1039,
-    EvLocallabSpotavoidmun = 1040,
+    //EvLocallabSpotavoidmun = 1040,
     Evlocallabcontthres = 1041,
     Evlocallabnorm = 1042,
     Evlocallabreparw = 1043,
