@@ -1,4 +1,5 @@
-/*
+/* -*- C++ -*-
+ *  
  *  This file is part of RawTherapee.
  *
  *  Copyright (c) 2004-2010 Gabor Horvath <hgabor@rawtherapee.com>
@@ -30,7 +31,6 @@
 #include "iimage.h"
 #include "imageformat.h"
 #include "procevents.h"
-#include "rawmetadatalocation.h"
 #include "settings.h"
 
 #include "../rtgui/threadutils.h"
@@ -51,12 +51,6 @@ class LUT;
 using LUTu = LUT<uint32_t>;
 
 class EditDataProvider;
-namespace rtexif
-{
-
-class TagDirectory;
-
-}
 
 namespace rtengine
 {
@@ -91,78 +85,56 @@ class FramesMetaData
 {
 
 public:
-    /** @return Returns the number of root Metadata */
-    virtual unsigned int getRootCount () const = 0;
     /** @return Returns the number of frame contained in the file based on Metadata */
-    virtual unsigned int getFrameCount () const = 0;
+    virtual unsigned int getFrameCount() const = 0;
 
     /** Checks the availability of exif metadata tags.
       * @return Returns true if image contains exif metadata tags */
-    virtual bool hasExif (unsigned int frame = 0) const = 0;
-    /** Returns the directory of exif metadata tags.
-      * @param root root number in the metadata tree
-      * @return The directory of exif metadata tags */
-    virtual rtexif::TagDirectory* getRootExifData (unsigned int root = 0) const = 0;
-    /** Returns the directory of exif metadata tags.
-      * @param frame frame number in the metadata tree
-      * @return The directory of exif metadata tags */
-    virtual rtexif::TagDirectory* getFrameExifData (unsigned int frame = 0) const = 0;
-    /** Returns the directory of exif metadata tags containing at least the 'Make' tag for the requested frame.
-      * If no usable metadata exist in the frame, send back the best TagDirectory describing the frame content.
-      * @param imgSource rawimage that we want the metadata from
-      * @param rawParams RawParams to select the frame number
-      * @return The directory of exif metadata tags containing at least the 'Make' tag */
-    virtual rtexif::TagDirectory* getBestExifData (ImageSource *imgSource, procparams::RAWParams *rawParams) const = 0;
-    /** Checks the availability of IPTC tags.
-      * @return Returns true if image contains IPTC tags */
-    virtual bool hasIPTC (unsigned int frame = 0) const = 0;
-    /** Returns the directory of IPTC tags.
-      * @return The directory of IPTC tags */
-    virtual procparams::IPTCPairs getIPTCData (unsigned int frame = 0) const = 0;
+    virtual bool hasExif() const = 0;
     /** @return a struct containing the date and time of the image */
-    virtual tm getDateTime (unsigned int frame = 0) const = 0;
+    virtual tm getDateTime() const = 0;
     /** @return a timestamp containing the date and time of the image */
-    virtual time_t getDateTimeAsTS(unsigned int frame = 0) const = 0;
+    virtual time_t getDateTimeAsTS() const = 0;
     /** @return the ISO of the image */
-    virtual int getISOSpeed (unsigned int frame = 0) const = 0;
+    virtual int getISOSpeed() const = 0;
     /** @return the F number of the image */
-    virtual double getFNumber  (unsigned int frame = 0) const = 0;
+    virtual double getFNumber() const = 0;
     /** @return the focal length used at the exposure */
-    virtual double getFocalLen (unsigned int frame = 0) const = 0;
+    virtual double getFocalLen() const = 0;
     /** @return the focal length in 35mm used at the exposure */
-    virtual double getFocalLen35mm (unsigned int frame = 0) const = 0;
+    virtual double getFocalLen35mm() const = 0;
     /** @return the focus distance in meters, 0=unknown, 10000=infinity */
-    virtual float getFocusDist (unsigned int frame = 0) const = 0;
+    virtual float getFocusDist() const = 0;
     /** @return the shutter speed */
-    virtual double getShutterSpeed (unsigned int frame = 0) const = 0;
+    virtual double getShutterSpeed() const = 0;
     /** @return the exposure compensation */
-    virtual double getExpComp (unsigned int frame = 0) const = 0;
+    virtual double getExpComp() const = 0;
     /** @return the maker of the camera */
-    virtual std::string getMake     (unsigned int frame = 0) const = 0;
+    virtual std::string getMake() const = 0;
     /** @return the model of the camera */
-    virtual std::string getModel    (unsigned int frame = 0) const = 0;
+    virtual std::string getModel() const = 0;
 
-    std::string getCamera   (unsigned int frame = 0) const
+    std::string getCamera() const
     {
-        return getMake(frame) + " " + getModel(frame);
+        return getMake() + " " + getModel();
     }
 
     /** @return the lens on the camera  */
-    virtual std::string getLens     (unsigned int frame = 0) const = 0;
+    virtual std::string getLens() const = 0;
     /** @return the orientation of the image */
-    virtual std::string getOrientation (unsigned int frame = 0) const = 0;
+    virtual std::string getOrientation() const = 0;
     /** @return the rating of the image */
-    virtual int getRating (unsigned int frame = 0) const = 0;
+    virtual int getRating() const = 0;
 
     /** @return true if the file is a PixelShift shot (Pentax and Sony bodies) */
     virtual bool getPixelShift () const = 0;
     /** @return false: not an HDR file ; true: single or multi-frame HDR file (e.g. Pentax HDR raw file or 32 bit float DNG file or Log compressed) */
-    virtual bool getHDR (unsigned int frame = 0) const = 0;
+    virtual bool getHDR() const = 0;
 
     /** @return false: not an HDR file ; true: single or multi-frame HDR file (e.g. Pentax HDR raw file or 32 bit float DNG file or Log compressed) */
-    virtual std::string getImageType (unsigned int frame) const = 0;
+    virtual std::string getImageType() const = 0;
     /** @return the sample format based on MetaData */
-    virtual IIOSampleFormat getSampleFormat (unsigned int frame = 0) const = 0;
+    virtual IIOSampleFormat getSampleFormat() const = 0;
 
     /** Functions to convert between floating point and string representation of shutter and aperture */
     static std::string apertureToString (double aperture);
@@ -183,7 +155,10 @@ public:
       * Use it only for raw files. In caseof jpgs and tiffs pass a NULL pointer.
       * @param firstFrameOnly must be true to get the MetaData of the first frame only, e.g. for a PixelShift file.
       * @return The metadata */
-    static FramesMetaData* fromFile (const Glib::ustring& fname, std::unique_ptr<RawMetaDataLocation> rml, bool firstFrameOnly = false);
+    static FramesMetaData* fromFile(const Glib::ustring& fname);
+
+    virtual Glib::ustring getFileName() const = 0;
+    virtual void getDimensions(int &w, int &h) const = 0;
 };
 
 /** This listener interface is used to indicate the progress of time consuming operations */
@@ -398,7 +373,7 @@ public :
     virtual void adapCamChanged(double cadap) = 0;
     virtual void ybCamChanged(int yb) = 0;
     virtual void wbCamChanged(double tem, double tin) = 0;
-    
+
 };
 
 class AutoChromaListener
@@ -438,9 +413,21 @@ public:
         double Tmax;
     };
 
+    struct locallabDenoiseLC  {
+        double highres;
+        double nres; 
+        double highres46;
+        double nres46; 
+        double Lhighres; 
+        double Lnres; 
+        double Lhighres46; 
+        double Lnres46;
+    };
+
     virtual ~LocallabListener() = default;
 //    virtual void refChanged(const std::vector<locallabRef> &ref, int selspot) = 0;
     virtual void minmaxChanged(const std::vector<locallabRetiMinMax> &minmax, int selspot) = 0;
+    virtual void denChanged(const std::vector<locallabDenoiseLC> &denlc, int selspot) = 0;
     virtual void logencodChanged(const float blackev, const float whiteev, const float sourceg, const float sourceab, const float targetg, const bool autocomput, const bool autocie, const float jz1) = 0;
     virtual void refChanged2(float *huerefp, float *chromarefp, float *lumarefp, float *fabrefp, int selspot) = 0;
 };
@@ -471,8 +458,15 @@ public:
 class AutoWBListener
 {
 public:
+    enum class AWBMode {
+        NONE,
+        RGB_GREY,
+        TEMP_CORRELATION_NON_RAW,
+        TEMP_CORRELATION_RAW,
+    };
+
     virtual ~AutoWBListener() = default;
-    virtual void WBChanged(double temp, double green, float studgood) = 0;
+    virtual void WBChanged(int met, double temp, double green, double rw, double gw, double bw, float temp0,  float delta, int bia, int dread, float studgood, float minchrom, int kmin, float histmin, float histmax, AWBMode aWBMode) = 0;
 };
 
 class FrameCountListener
@@ -493,7 +487,7 @@ class ImageTypeListener
 {
 public:
     virtual ~ImageTypeListener() = default;
-    virtual void imageTypeChanged(bool isRaw, bool isBayer, bool isXtrans, bool is_Mono = false) = 0;
+    virtual void imageTypeChanged(bool isRaw, bool isBayer, bool isXtrans, bool is_Mono = false, bool isGainMapSupported = false) = 0;
 };
 
 class AutoContrastListener
@@ -619,11 +613,11 @@ public:
       * @return a pointer to the Crop object that handles the image data trough its own pipeline */
     virtual DetailedCrop* createCrop  (::EditDataProvider *editDataProvider, bool isDetailWindow) = 0;
 
-    virtual bool        getAutoWB   (double& temp, double& green, double equal, double tempBias) = 0;
-    virtual void        getCamWB    (double& temp, double& green) = 0;
+    virtual bool        getAutoWB   (double& temp, double& green, double equal, StandardObserver observer, double tempBias) = 0;
+    virtual void        getCamWB    (double& temp, double& green, StandardObserver observer) = 0;
     virtual void        getSpotWB  (int x, int y, int rectSize, double& temp, double& green) = 0;
     virtual bool        getFilmNegativeSpot(int x, int y, int spotSize, procparams::FilmNegativeParams::RGB &refInput, procparams::FilmNegativeParams::RGB &refOutput) = 0;
-    
+
     virtual void        getAutoCrop (double ratio, int &x, int &y, int &w, int &h) = 0;
 
     virtual void        saveInputICCReference (const Glib::ustring& fname, bool apply_wb) = 0;
