@@ -46,7 +46,7 @@ LFModifier::operator bool() const
 }
 
 
-void LFModifier::correctDistortion(double &x, double &y, int cx, int cy, double scale) const
+void LFModifier::correctDistortion(double &x, double &y, int cx, int cy) const
 {
     if (!data_) {
         return;
@@ -67,8 +67,6 @@ void LFModifier::correctDistortion(double &x, double &y, int cx, int cy, double
         x -= cx;
         y -= cy;
     }
-    x *= scale;
-    y *= scale;
 }
 
 
@@ -440,7 +438,7 @@ std::vector<LFLens> LFDatabase::getLenses() const
 LFCamera LFDatabase::findCamera(const Glib::ustring &make, const Glib::ustring &model) const
 {
     LFCamera ret;
-    if (data_) {
+    if (data_ && !make.empty()) {
         MyMutex::MyLock lock(lfDBMutex);
         auto found = data_->FindCamerasExt(make.c_str(), model.c_str());
         if (found) {
@@ -455,7 +453,7 @@ LFCamera LFDatabase::findCamera(const Glib::ustring &make, const Glib::ustring &
 LFLens LFDatabase::findLens(const LFCamera &camera, const Glib::ustring &name) const
 {
     LFLens ret;
-    if (data_) {
+    if (data_ && !name.empty()) {
         MyMutex::MyLock lock(lfDBMutex);
         auto found = data_->FindLenses(camera.data_, nullptr, name.c_str());
         for (size_t pos = 0; !found && pos < name.size(); ) {
