@@ -16,6 +16,8 @@
  *  You should have received a copy of the GNU General Public License
  *  along with RawTherapee.  If not, see <https://www.gnu.org/licenses/>.
  */
+#include <algorithm>
+#include <array>
 #include <clocale>
 
 #include <lcms2.h>
@@ -65,6 +67,84 @@ bool checkRawImageThumb (const rtengine::RawImage& raw_image)
     return raw_image.get_thumbOffset() + length <= raw_image.get_file()->size;
 }
 
+/**
+ * Apply the black level adjustments in the processing parameters.
+ *
+ * @param cblack The original black levels that will be modified.
+ * @param sensorType Sensor type.
+ * @param rawParams Subset of processing parameters for raw data.
+ */
+void adjustBlackLevels(float cblack[4], rtengine::eSensorType sensorType, const rtengine::RAWParams *rawParams)
+{
+    if (!rawParams) {
+        return;
+    }
+
+    std::array<float, 4> black_adjust{0.f, 0.f, 0.f, 0.f};
+
+    switch (sensorType) {
+        case rtengine::eSensorType::ST_BAYER:
+        case rtengine::eSensorType::ST_FOVEON:
+            black_adjust[0] = static_cast<float>(rawParams->bayersensor.black1); // R
+            black_adjust[1] = static_cast<float>(rawParams->bayersensor.black0); // G1
+            black_adjust[2] = static_cast<float>(rawParams->bayersensor.black2); // B
+            black_adjust[3] = static_cast<float>(rawParams->bayersensor.black3); // G2
+            break;
+        case rtengine::eSensorType::ST_FUJI_XTRANS:
+            black_adjust[0] = static_cast<float>(rawParams->xtranssensor.blackred);
+            black_adjust[1] = static_cast<float>(rawParams->xtranssensor.blackgreen);
+            black_adjust[2] = static_cast<float>(rawParams->xtranssensor.blackblue);
+            black_adjust[3] = static_cast<float>(rawParams->xtranssensor.blackgreen);
+            break;
+        case rtengine::eSensorType::ST_NONE:
+            break;
+    }
+
+    for (unsigned int i = 0; i < black_adjust.size(); i++) {
+        cblack[i] = std::max(0.f, cblack[i] + black_adjust[i]);
+    }
+}
+
+/**
+ * Calculate the new scale multipliers based on new black levels.
+ *
+ * @param scale_mul The original scale multipliers to be adjusted.
+ * @param pre_mul Pre-multipliers.
+ * @param c_black Updated black levels.
+ * @param isMono Is the image using mono demosaicing?
+ * @param ri Pointer to the raw image.
+ */
+void calculate_scale_mul(float scale_mul[4], const float pre_mul_[4], const float c_black[4], bool isMono, const rtengine::RawImage *ri)
+{
+    std::array<float, 4> c_white;
+
+    for (unsigned int i = 0; i < c_white.size(); ++i) {
+        c_white[i] = static_cast<float>(ri->get_white(i));
+    }
+
+    if (isMono || ri->get_colors() == 1) {
+        for (int c = 0; c < 4; c++) {
+            scale_mul[c] = 65535.f / (c_white[c] - c_black[c]);
+        }
+    } else {
+        std::array<float, 4> pre_mul;
+
+        for (int c = 0; c < 4; c++) {
+            pre_mul[c] = pre_mul_[c];
+        }
+
+        if (pre_mul[3] == 0) {
+            pre_mul[3] = pre_mul[1]; // G2 == G1
+        }
+
+        float maxpremul = std::max(std::max(std::max(pre_mul[0], pre_mul[1]), pre_mul[2]), pre_mul[3]);
+
+        for (int c = 0; c < 4; c++) {
+            scale_mul[c] = (pre_mul[c] / maxpremul) * 65535.f / (c_white[c] - c_black[c]);
+        }
+    }
+}
+
 
 void scale_colors (rtengine::RawImage *ri, float scale_mul[4], float cblack[4], bool multiThread)
 {
@@ -194,7 +274,7 @@ namespace rtengine
 
 using namespace procparams;
 
-Thumbnail* Thumbnail::loadFromImage (const Glib::ustring& fname, int &w, int &h, int fixwh, double wbEq, bool inspectorMode)
+Thumbnail* Thumbnail::loadFromImage (const Glib::ustring& fname, int &w, int &h, int fixwh, double wbEq, StandardObserver wbObserver, bool inspectorMode)
 {
 
     StdImageSource imgSrc;
@@ -310,8 +390,9 @@ Thumbnail* Thumbnail::loadFromImage (const Glib::ustring& fname, int &w, int &h,
             tpp->blueAWBMul  = avg_b / double (n);
             tpp->wbEqual = wbEq;
             tpp->wbTempBias = 0.0;
+            tpp->wbObserver = wbObserver;
 
-            cTemp.mul2temp (tpp->redAWBMul, tpp->greenAWBMul, tpp->blueAWBMul, tpp->wbEqual, tpp->autoWBTemp, tpp->autoWBGreen);
+            cTemp.mul2temp (tpp->redAWBMul, tpp->greenAWBMul, tpp->blueAWBMul, tpp->wbEqual, tpp->wbObserver, tpp->autoWBTemp, tpp->autoWBGreen);
         }
 
         tpp->init ();
@@ -323,10 +404,10 @@ Thumbnail* Thumbnail::loadFromImage (const Glib::ustring& fname, int &w, int &h,
 
 namespace {
 
-Image8 *load_inspector_mode(const Glib::ustring &fname, RawMetaDataLocation &rml, eSensorType &sensorType, int &w, int &h)
+Image8 *load_inspector_mode(const Glib::ustring &fname, eSensorType &sensorType, int &w, int &h)
 {
     BENCHFUN
-    
+
     RawImageSource src;
     int err = src.load(fname, true);
     if (err) {
@@ -335,7 +416,7 @@ Image8 *load_inspector_mode(const Glib::ustring &fname, RawMetaDataLocation &rml
 
     src.getFullSize(w, h);
     sensorType = src.getSensorType();
-    
+
     ProcParams neutral;
     neutral.raw.bayersensor.method = RAWParams::BayerSensor::getMethodString(RAWParams::BayerSensor::Method::FAST);
     neutral.raw.xtranssensor.method = RAWParams::XTransSensor::getMethodString(RAWParams::XTransSensor::Method::FAST);
@@ -379,7 +460,7 @@ Image8 *load_inspector_mode(const Glib::ustring &fname, RawMetaDataLocation &rml
 
 } // namespace
 
-Thumbnail* Thumbnail::loadQuickFromRaw (const Glib::ustring& fname, RawMetaDataLocation& rml, eSensorType &sensorType, int &w, int &h, int fixwh, bool rotate, bool inspectorMode, bool forHistogramMatching)
+Thumbnail* Thumbnail::loadQuickFromRaw (const Glib::ustring& fname, eSensorType &sensorType, int &w, int &h, int fixwh, bool rotate, bool inspectorMode, bool forHistogramMatching)
 {
     Thumbnail* tpp = new Thumbnail ();
     tpp->isRaw = 1;
@@ -389,7 +470,7 @@ Thumbnail* Thumbnail::loadQuickFromRaw (const Glib::ustring& fname, RawMetaDataL
     tpp->colorMatrix[2][2] = 1.0;
 
     if (inspectorMode && !forHistogramMatching && settings->thumbnail_inspector_mode == Settings::ThumbnailInspectorMode::RAW) {
-        Image8 *img = load_inspector_mode(fname, rml, sensorType, w, h);
+        Image8 *img = load_inspector_mode(fname, sensorType, w, h);
         if (!img) {
             delete tpp;
             return nullptr;
@@ -400,7 +481,7 @@ Thumbnail* Thumbnail::loadQuickFromRaw (const Glib::ustring& fname, RawMetaDataL
 
         return tpp;
     }
-    
+
     RawImage *ri = new RawImage (fname);
     unsigned int imageNum = 0;
     int r = ri->loadRaw (false, imageNum, false);
@@ -414,10 +495,6 @@ Thumbnail* Thumbnail::loadQuickFromRaw (const Glib::ustring& fname, RawMetaDataL
 
     sensorType = ri->getSensorType();
 
-    rml.exifBase = ri->get_exifBase();
-    rml.ciffBase = ri->get_ciffBase();
-    rml.ciffLength = ri->get_ciffLen();
-
     Image8* img = new Image8 ();
     // No sample format detection occurred earlier, so we set them here,
     // as they are mandatory for the setScanline method
@@ -457,8 +534,8 @@ Thumbnail* Thumbnail::loadQuickFromRaw (const Glib::ustring& fname, RawMetaDataL
         if (!forHistogramMatching && settings->thumbnail_inspector_mode == Settings::ThumbnailInspectorMode::RAW_IF_NOT_JPEG_FULLSIZE && float(std::max(w, h))/float(std::max(ri->get_width(), ri->get_height())) < 0.9f) {
             delete img;
             delete ri;
-            
-            img = load_inspector_mode(fname, rml, sensorType, w, h);
+
+            img = load_inspector_mode(fname, sensorType, w, h);
             if (!img) {
                 delete tpp;
                 return nullptr;
@@ -466,7 +543,7 @@ Thumbnail* Thumbnail::loadQuickFromRaw (const Glib::ustring& fname, RawMetaDataL
 
             tpp->scale = 1.;
             tpp->thumbImg = img;
-            
+
             return tpp;
         }
     } else {
@@ -522,28 +599,7 @@ Thumbnail* Thumbnail::loadQuickFromRaw (const Glib::ustring& fname, RawMetaDataL
 #define FISGREEN(filter,row,col) \
     ((filter >> ((((row) << 1 & 14) + ((col) & 1)) << 1) & 3)==1 || !filter)
 
-RawMetaDataLocation Thumbnail::loadMetaDataFromRaw (const Glib::ustring& fname)
-{
-    RawMetaDataLocation rml;
-    rml.exifBase = -1;
-    rml.ciffBase = -1;
-    rml.ciffLength = -1;
-
-    RawImage ri (fname);
-    unsigned int imageNum = 0;
-
-    int r = ri.loadRaw (false, imageNum);
-
-    if ( !r ) {
-        rml.exifBase = ri.get_exifBase();
-        rml.ciffBase = ri.get_ciffBase();
-        rml.ciffLength = ri.get_ciffLen();
-    }
-
-    return rml;
-}
-
-Thumbnail* Thumbnail::loadFromRaw (const Glib::ustring& fname, RawMetaDataLocation& rml, eSensorType &sensorType, int &w, int &h, int fixwh, double wbEq, bool rotate, bool forHistogramMatching)
+Thumbnail* Thumbnail::loadFromRaw (const Glib::ustring& fname, eSensorType &sensorType, int &w, int &h, int fixwh, double wbEq, StandardObserver wbObserver, bool rotate, const RAWParams *rawParams, bool forHistogramMatching)
 {
     RawImage *ri = new RawImage (fname);
     unsigned int tempImageNum = 0;
@@ -586,16 +642,20 @@ Thumbnail* Thumbnail::loadFromRaw (const Glib::ustring& fname, RawMetaDataLocati
     tpp->greenMultiplier = ri->get_pre_mul (1);
     tpp->blueMultiplier = ri->get_pre_mul (2);
 
+    bool isMono =
+        rawParams &&
+        ((ri->getSensorType() == ST_FUJI_XTRANS &&
+             rawParams->xtranssensor.method == RAWParams::XTransSensor::getMethodString(RAWParams::XTransSensor::Method::MONO)) ||
+            (ri->getSensorType() == ST_BAYER &&
+                rawParams->bayersensor.method == RAWParams::BayerSensor::getMethodString(RAWParams::BayerSensor::Method::MONO)));
     float pre_mul[4], scale_mul[4], cblack[4];
     ri->get_colorsCoeff (pre_mul, scale_mul, cblack, false);
+    adjustBlackLevels(cblack, sensorType, rawParams);
+    calculate_scale_mul(scale_mul, pre_mul, cblack, isMono, ri);
     scale_colors (ri, scale_mul, cblack, forHistogramMatching); // enable multithreading when forHistogramMatching is true
 
     ri->pre_interpolate();
 
-    rml.exifBase = ri->get_exifBase();
-    rml.ciffBase = ri->get_ciffBase();
-    rml.ciffLength = ri->get_ciffLen();
-
     tpp->camwbRed = tpp->redMultiplier / pre_mul[0]; //ri->get_pre_mul(0);
     tpp->camwbGreen = tpp->greenMultiplier / pre_mul[1]; //ri->get_pre_mul(1);
     tpp->camwbBlue = tpp->blueMultiplier / pre_mul[2]; //ri->get_pre_mul(2);
@@ -982,9 +1042,10 @@ Thumbnail* Thumbnail::loadFromRaw (const Glib::ustring& fname, RawMetaDataLocati
         tpp->blueAWBMul  = ri->get_rgb_cam (2, 0) * reds + ri->get_rgb_cam (2, 1) * greens + ri->get_rgb_cam (2, 2) * blues;
         tpp->wbEqual = wbEq;
         tpp->wbTempBias = 0.0;
+        tpp->wbObserver = wbObserver;
 
         ColorTemp cTemp;
-        cTemp.mul2temp (tpp->redAWBMul, tpp->greenAWBMul, tpp->blueAWBMul, tpp->wbEqual, tpp->autoWBTemp, tpp->autoWBGreen);
+        cTemp.mul2temp (tpp->redAWBMul, tpp->greenAWBMul, tpp->blueAWBMul, tpp->wbEqual, tpp->wbObserver, tpp->autoWBTemp, tpp->autoWBGreen);
     }
 
     if (rotate && ri->get_rotateDegree() > 0) {
@@ -997,6 +1058,9 @@ Thumbnail* Thumbnail::loadFromRaw (const Glib::ustring& fname, RawMetaDataLocati
         }
 
     tpp->init();
+
+    RawImageSource::computeFullSize(ri, TR_NONE, tpp->full_width, tpp->full_height);
+
     delete ri;
     return tpp;
 }
@@ -1054,7 +1118,9 @@ Thumbnail::Thumbnail () :
     gammaCorrected (false),
     colorMatrix{},
     scaleGain (1.0),
-    isRaw (true)
+    isRaw (true),
+    full_width(-1),
+    full_height(-1)
 {
 }
 
@@ -1107,32 +1173,25 @@ IImage8* Thumbnail::quickProcessImage (const procparams::ProcParams& params, int
 // Full thumbnail processing, second stage if complete profile exists
 IImage8* Thumbnail::processImage (const procparams::ProcParams& params, eSensorType sensorType, int rheight, TypeInterpolation interp, const FramesMetaData *metadata, double& myscale, bool forMonitor, bool forHistogramMatching)
 {
-    unsigned int imgNum = 0;
-    if (isRaw) {
-        if (sensorType == ST_BAYER) {
-            imgNum = rtengine::LIM<unsigned int>(params.raw.bayersensor.imageNum, 0, metadata->getFrameCount() - 1);
-        } else if (sensorType == ST_FUJI_XTRANS) {
-            //imgNum = rtengine::LIM<unsigned int>(params.raw.xtranssensor.imageNum, 0, metadata->getFrameCount() - 1)
-        }
-    }
-    std::string camName = metadata->getCamera(imgNum);
-    float shutter = metadata->getShutterSpeed(imgNum);
-    float fnumber = metadata->getFNumber(imgNum);
-    float iso = metadata->getISOSpeed(imgNum);
-    float fcomp = metadata->getExpComp(imgNum);
-    
-    // check if the WB's equalizer value has changed
-    if (wbEqual < (params.wb.equal - 5e-4) || wbEqual > (params.wb.equal + 5e-4) || wbTempBias < (params.wb.tempBias - 5e-4) || wbTempBias > (params.wb.tempBias + 5e-4)) {
+    const std::string camName = metadata->getCamera();
+    const float shutter = metadata->getShutterSpeed();
+    const float fnumber = metadata->getFNumber();
+    const float iso = metadata->getISOSpeed();
+    const float fcomp = metadata->getExpComp();
+
+    // check if the WB's equalizer, temperature bias, or observer value has changed
+    if (wbEqual < (params.wb.equal - 5e-4) || wbEqual > (params.wb.equal + 5e-4) || wbTempBias < (params.wb.tempBias - 5e-4) || wbTempBias > (params.wb.tempBias + 5e-4) || wbObserver != params.wb.observer) {
         wbEqual = params.wb.equal;
         wbTempBias = params.wb.tempBias;
+        wbObserver = params.wb.observer;
         // recompute the autoWB
         ColorTemp cTemp;
-        cTemp.mul2temp (redAWBMul, greenAWBMul, blueAWBMul, wbEqual, autoWBTemp, autoWBGreen);
+        cTemp.mul2temp (redAWBMul, greenAWBMul, blueAWBMul, wbEqual, wbObserver, autoWBTemp, autoWBGreen);
         autoWBTemp += autoWBTemp * wbTempBias;
     }
 
     // compute WB multipliers
-    ColorTemp currWB = ColorTemp (params.wb.temperature, params.wb.green, params.wb.equal, params.wb.method);
+    ColorTemp currWB = ColorTemp (params.wb.temperature, params.wb.green, params.wb.equal, params.wb.method, params.wb.observer);
 
     if (!params.wb.enabled) {
         currWB = ColorTemp();
@@ -1141,9 +1200,21 @@ IImage8* Thumbnail::processImage (const procparams::ProcParams& params, eSensorT
         double cam_r = colorMatrix[0][0] * camwbRed + colorMatrix[0][1] * camwbGreen + colorMatrix[0][2] * camwbBlue;
         double cam_g = colorMatrix[1][0] * camwbRed + colorMatrix[1][1] * camwbGreen + colorMatrix[1][2] * camwbBlue;
         double cam_b = colorMatrix[2][0] * camwbRed + colorMatrix[2][1] * camwbGreen + colorMatrix[2][2] * camwbBlue;
-        currWB = ColorTemp (cam_r, cam_g, cam_b, params.wb.equal);
+        currWB = ColorTemp (cam_r, cam_g, cam_b, params.wb.equal, params.wb.observer);
     } else if (params.wb.method == "autold") {
-        currWB = ColorTemp (autoWBTemp, autoWBGreen, wbEqual, "Custom");
+        if (params.wb.compat_version == 1 && !isRaw) {
+            // RGB grey compatibility version 1 used the identity multipliers
+            // plus temperature bias for non-raw files.
+            currWB.update(1., 1., 1., params.wb.equal, params.wb.observer, params.wb.tempBias);
+        } else {
+            currWB = ColorTemp(autoWBTemp, autoWBGreen, wbEqual, "Custom", wbObserver);
+        }
+    } else if (params.wb.method == "autitcgreen") {
+        if (params.wb.compat_version == 1 && !isRaw) {
+            currWB = ColorTemp(5000., 1., 1., params.wb.method, StandardObserver::TEN_DEGREES);
+        } else {
+            // TODO: Temperature correlation AWB.
+        }
     }
 
     double rm, gm, bm;
@@ -1226,7 +1297,7 @@ IImage8* Thumbnail::processImage (const procparams::ProcParams& params, eSensorT
             float red = baseImg->r (i, j) * rmi;
             float green = baseImg->g (i, j) * gmi;
             float blue = baseImg->b (i, j) * bmi;
-            
+
             // avoid magenta highlights if highlight recovery is enabled
             if (params.toneCurve.hrenabled && red > MAXVALF && blue > MAXVALF) {
                 baseImg->r(i, j) = baseImg->g(i, j) = baseImg->b(i, j) = CLIP((red + green + blue) / 3.f);
@@ -1272,8 +1343,8 @@ IImage8* Thumbnail::processImage (const procparams::ProcParams& params, eSensorT
     ipf.firstAnalysis (baseImg, params, hist16);
 
     ipf.dehaze(baseImg, params.dehaze);
-    ipf.ToneMapFattal02(baseImg, params.fattal, 3, 0, nullptr, 0, 0, 0);
-    
+    ipf.ToneMapFattal02(baseImg, params.fattal, 3, 0, nullptr, 0, 0, 0, false);
+
     // perform transform
     int origFW;
     int origFH;
@@ -1424,7 +1495,7 @@ IImage8* Thumbnail::processImage (const procparams::ProcParams& params, eSensorT
             }
     }
 
-    
+
     // luminance processing
 //  ipf.EPDToneMap(labView,0,6);
 
@@ -1463,6 +1534,50 @@ IImage8* Thumbnail::processImage (const procparams::ProcParams& params, eSensorT
 
     ipf.softLight(labView, params.softlight);
 
+    if (params.icm.workingTRC != ColorManagementParams::WorkingTrc::NONE) {
+        const int GW = labView->W;
+        const int GH = labView->H;
+        std::unique_ptr<LabImage> provis;
+        const float pres = 0.01f * params.icm.preser;
+        if (pres > 0.f && params.icm.wprim != ColorManagementParams::Primaries::DEFAULT) {
+            provis.reset(new LabImage(GW, GH));
+            provis->CopyFrom(labView);
+        }
+
+        const std::unique_ptr<Imagefloat> tmpImage1(new Imagefloat(GW, GH));
+
+        ipf.lab2rgb(*labView, *tmpImage1, params.icm.workingProfile);
+
+        const float gamtone = params.icm.workingTRCGamma;
+        const float slotone = params.icm.workingTRCSlope;
+
+        int illum = toUnderlying(params.icm.will);
+        const int prim = toUnderlying(params.icm.wprim);
+
+        Glib::ustring prof = params.icm.workingProfile;
+
+        cmsHTRANSFORM dummy = nullptr;
+        int ill = 0;
+        ipf.workingtrc(tmpImage1.get(), tmpImage1.get(), GW, GH, -5, prof, 2.4, 12.92310, ill, 0, dummy, true, false, false);
+        ipf.workingtrc(tmpImage1.get(), tmpImage1.get(), GW, GH, 5, prof, gamtone, slotone, illum, prim, dummy, false, true, true);
+
+        ipf.rgb2lab(*tmpImage1, *labView, params.icm.workingProfile);
+        // labView and provis
+        if(provis) {
+            ipf.preserv(labView, provis.get(), GW, GH);
+        }
+        if(params.icm.fbw) {
+#ifdef _OPENMP
+        #pragma omp parallel for
+#endif
+        for (int x = 0; x < GH; x++)
+            for (int y = 0; y < GW; y++) {
+                labView->a[x][y] = 0.f;
+                labView->b[x][y] = 0.f;
+            }
+        }
+
+    }
 
     if (params.colorappearance.enabled) {
         CurveFactory::curveLightBrightColor (
@@ -1588,27 +1703,28 @@ void Thumbnail::getDimensions (int& w, int& h, double& scaleFac)
     }
 }
 
-void Thumbnail::getCamWB (double& temp, double& green)
+void Thumbnail::getCamWB (double& temp, double& green, StandardObserver observer)
 {
 
     double cam_r = colorMatrix[0][0] * camwbRed + colorMatrix[0][1] * camwbGreen + colorMatrix[0][2] * camwbBlue;
     double cam_g = colorMatrix[1][0] * camwbRed + colorMatrix[1][1] * camwbGreen + colorMatrix[1][2] * camwbBlue;
     double cam_b = colorMatrix[2][0] * camwbRed + colorMatrix[2][1] * camwbGreen + colorMatrix[2][2] * camwbBlue;
-    ColorTemp currWB = ColorTemp (cam_r, cam_g, cam_b, 1.0);  // we do not take the equalizer into account here, because we want camera's WB
+    ColorTemp currWB = ColorTemp (cam_r, cam_g, cam_b, 1.0, observer);  // we do not take the equalizer into account here, because we want camera's WB
     temp = currWB.getTemp ();
     green = currWB.getGreen ();
 }
 
-void Thumbnail::getAutoWB (double& temp, double& green, double equal, double tempBias)
+void Thumbnail::getAutoWB (double& temp, double& green, double equal, double tempBias, StandardObserver observer)
 {
 
-    if (equal != wbEqual || tempBias != wbTempBias) {
+    if (equal != wbEqual || tempBias != wbTempBias || observer != wbObserver) {
         // compute the values depending on equal
         ColorTemp cTemp;
         wbEqual = equal;
         wbTempBias = tempBias;
+        wbObserver = observer;
         // compute autoWBTemp and autoWBGreen
-        cTemp.mul2temp (redAWBMul, greenAWBMul, blueAWBMul, wbEqual, autoWBTemp, autoWBGreen);
+        cTemp.mul2temp (redAWBMul, greenAWBMul, blueAWBMul, wbEqual, wbObserver, autoWBTemp, autoWBGreen);
         autoWBTemp += autoWBTemp * tempBias;
     }
 
@@ -1665,7 +1781,7 @@ void Thumbnail::getSpotWB (const procparams::ProcParams& params, int xp, int yp,
     double gm = colorMatrix[1][0] * reds + colorMatrix[1][1] * greens + colorMatrix[1][2] * blues;
     double bm = colorMatrix[2][0] * reds + colorMatrix[2][1] * greens + colorMatrix[2][2] * blues;
 
-    ColorTemp ct (rm, gm, bm, params.wb.equal);
+    ColorTemp ct (rm, gm, bm, params.wb.equal, params.wb.observer);
     rtemp = ct.getTemp ();
     rgreen = ct.getGreen ();
 }
@@ -2167,7 +2283,7 @@ bool Thumbnail::readData  (const Glib::ustring& fname)
                         colorMatrix[i][j] = cm[ix++];
                     }
             }
-            
+
             if (keyFile.has_key ("LiveThumbData", "ScaleGain")) {
                 scaleGain           = keyFile.get_double ("LiveThumbData", "ScaleGain");
             }
