@@ -32,6 +32,7 @@
 #include "imagesource.h"
 #include "improcfun.h"
 #include "labimage.h"
+#include "metadata.h"
 #include "mytime.h"
 #include "processingjob.h"
 #include "procparams.h"
@@ -261,7 +262,7 @@ private:
             pl->setProgress(0.40);
         }
 
-        imgsrc->HLRecovery_Global(params.toneCurve);
+        // imgsrc->HLRecovery_Global(params.toneCurve);
 
 
         if (pl) {
@@ -269,18 +270,87 @@ private:
         }
 
         // set the color temperature
-        currWB = ColorTemp(params.wb.temperature, params.wb.green, params.wb.equal, params.wb.method);
+        currWB = ColorTemp(params.wb.temperature, params.wb.green, params.wb.equal, params.wb.method, params.wb.observer);
+        ColorTemp currWBitc;
+        if (params.wb.method == "autitcgreen"  && flush) {
+            imgsrc->getrgbloc(0, 0, fh, fw, 0, 0, fh, fw, params.wb);
+        }
+        const bool autowb = (params.wb.method == "autitcgreen" && imgsrc->isRAW() && flush);
+        ColorTemp autoWB;
+        int dread = 0;
+        int bia = 1;
+        float studgood = 1000.f;
+        int nocam = 0;
+        int kcam = 0;
+        float minchrom = 1000.f;
+        float delta = 0.f;
+        int kmin  = 20;
+        float minhist = 1000000000.f;
+        float maxhist = -1000.f;
+        double greenitc = 1.;
+        float temp0 = 5000.f;
+        bool extra = false;
+        bool forcewbgrey = false;
 
         if (!params.wb.enabled) {
             currWB = ColorTemp();
-        } else if (params.wb.method == "Camera") {
+        } else if (params.wb.method == "Camera" || (params.wb.method == "autitcgreen" && params.wb.compat_version >= 2 && !imgsrc->isRAW() && flush)) {//Use also Camera settings for Temperature correlation and TIF/Jpg
             currWB = imgsrc->getWB();
-        } else if (params.wb.method == "autold") {
+        } else if (params.wb.method == "autold") {//for Auto RGB
             double rm, gm, bm;
-            imgsrc->getAutoWBMultipliers(rm, gm, bm);
-            currWB.update(rm, gm, bm, params.wb.equal, params.wb.tempBias);
-        }
+            if (params.wb.compat_version == 1 && !imgsrc->isRAW()) {
+                // RGB grey compatibility version 1 used the identity
+                // multipliers plus temperature bias for non-raw files.
+                rm = gm = bm = 1.;
+            } else {
+                imgsrc->getAutoWBMultipliers(rm, gm, bm);
+            }
+            currWB.update(rm, gm, bm, params.wb.equal, params.wb.observer, params.wb.tempBias);
 
+        } else if (autowb) {//for auto Itcwb - flush to enable only when batch only with Raw files
+        //code similar to that present in improccoordinator.cc
+                double rm;
+                double gm;
+                double bm;
+                imgsrc->getAutoWBMultipliersItcGreen(
+                    params,
+                    forcewbgrey,
+                    kcam,
+                    greenitc,
+                    extra,
+                    temp0,
+                    delta,
+                    bia,
+                    dread,
+                    nocam,
+                    studgood,
+                    minchrom,
+                    kmin,
+                    minhist,
+                    maxhist,
+                    fh,
+                    fw,
+                    currWB,
+                    0,
+                    0.,
+                    false,
+                    autoWB,
+                    rm,
+                    gm,
+                    bm);
+
+                currWB = autoWB;
+        } else if (params.wb.method == "autitcgreen" && params.wb.compat_version == 1 && !imgsrc->isRAW() && flush) {
+            // ITCWB compatibility version 1 used 5000 K and observer 10 degrees
+            // for non-raw files.
+            currWB = ColorTemp(5000., 1., 1., params.wb.method, StandardObserver::TEN_DEGREES);
+            currWB.convertObserver(params.wb.observer);
+            params.wb.temperature = currWB.getTemp();
+            params.wb.green = currWB.getGreen();
+            params.wb.equal = currWB.getEqual();
+        }
+        //end WB auto
+        
         calclum = nullptr ;
         params.dirpyrDenoise.getCurves(noiseLCurve, noiseCCurve);
         autoNR = (float) settings->nrauto;//
@@ -781,7 +851,7 @@ private:
 
         if (params.toneCurve.histmatching) {
             if (!params.toneCurve.fromHistMatching) {
-                imgsrc->getAutoMatchedToneCurve(params.icm, params.toneCurve.curve);
+                imgsrc->getAutoMatchedToneCurve(params.icm, params.raw, params.wb.observer, params.toneCurve.curve);
             }
 
             if (params.toneCurve.autoexp) {
@@ -896,7 +966,7 @@ private:
         ipf.firstAnalysis(baseImg, params, hist16);
 
         ipf.dehaze(baseImg, params.dehaze);
-        ipf.ToneMapFattal02(baseImg, params.fattal, 3, 0, nullptr, 0, 0, 0);
+        ipf.ToneMapFattal02(baseImg, params.fattal, 3, 0, nullptr, 0, 0, 0, false);
 
         // perform transform (excepted resizing)
         if (ipf.needsTransform(fw, fh, imgsrc->getRotateDegree(), imgsrc->getMetaData())) {
@@ -928,7 +998,7 @@ private:
 			if(params.locallab.spots.at(sp).expsharp  && params.dirpyrequalizer.cbdlMethod == "bef") {
 				if(params.locallab.spots.at(sp).shardamping < 1) {
 					params.locallab.spots.at(sp).shardamping = 1;
-				}				
+				}
 			}
 		}
 
@@ -947,10 +1017,10 @@ private:
 
         if (params.locallab.enabled && params.locallab.spots.size() > 0) {
             ipf.rgb2lab(*baseImg, *labView, params.icm.workingProfile);
-            
+
             MyTime t1, t2;
             t1.set();
-            
+
             const std::unique_ptr<LabImage> reservView(new LabImage(*labView, true));
             const std::unique_ptr<LabImage> lastorigView(new LabImage(*labView, true));
             std::unique_ptr<LabImage> savenormtmView;
@@ -1002,7 +1072,7 @@ private:
             LocLLmaskCurve locllmas_Curve;
             LocHHmaskCurve lochhmas_Curve;
             LocHHmaskCurve lochhhmas_Curve;
-            
+
             LocwavCurve loclmasCurveblwav;
             LocwavCurve loclmasCurvecolwav;
             LocwavCurve loclmasCurve_wav;
@@ -1094,7 +1164,7 @@ private:
                 const bool lcmascieutili = locccmascieCurve.Set(params.locallab.spots.at(sp).CCmaskciecurve);
                 const bool llmascieutili = locllmascieCurve.Set(params.locallab.spots.at(sp).LLmaskciecurve);
                 const bool lhmascieutili = lochhmascieCurve.Set(params.locallab.spots.at(sp).HHmaskciecurve);
-                
+
                 const bool lcmas_utili = locccmas_Curve.Set(params.locallab.spots.at(sp).CCmask_curve);
                 const bool llmas_utili = locllmas_Curve.Set(params.locallab.spots.at(sp).LLmask_curve);
                 const bool lhmas_utili = lochhmas_Curve.Set(params.locallab.spots.at(sp).HHmask_curve);
@@ -1158,7 +1228,7 @@ private:
                 float meanretie;
                 float stdretie;
                 float fab = 1.f;
-                
+
                 if (params.locallab.spots.at(sp).spotMethod == "exc") {
                     ipf.calc_ref(sp, reservView.get(), reservView.get(), 0, 0, fw, fh, 1, huerefblu, chromarefblu, lumarefblu, huere, chromare, lumare, sobelre, avge, locwavCurveden, locwavdenutili);
                 } else {
@@ -1175,10 +1245,18 @@ private:
                 float Tsigma;
                 float Tmin;
                 float Tmax;
+                float highresi = 0.f;
+                float nresi = 0.f;
+                float highresi46 =0.f;
+                float nresi46 = 0.f;
+                float Lhighresi = 0.f;
+                float Lnresi = 0.f;
+                float Lhighresi46 = 0.f;
+                float Lnresi46 = 0.f;
 
                 // No Locallab mask is shown in exported picture
-                ipf.Lab_Local(2, sp, shbuffer, labView, labView, reservView.get(), savenormtmView.get(), savenormretiView.get(), lastorigView.get(), fw, fh, 0, 0, fw, fh,  1, locRETgainCurve, locRETtransCurve, 
-                        lllocalcurve, locallutili, 
+                ipf.Lab_Local(2, sp, shbuffer, labView, labView, reservView.get(), savenormtmView.get(), savenormretiView.get(), lastorigView.get(), fw, fh, 0, 0, fw, fh,  1, locRETgainCurve, locRETtransCurve,
+                        lllocalcurve, locallutili,
                         cllocalcurve, localclutili,
                         lclocalcurve, locallcutili,
                         loclhCurve, lochhCurve, locchCurve,
@@ -1195,12 +1273,12 @@ private:
                         lmaskloglocalcurve, localmasklogutili,
                         lmasklocal_curve, localmask_utili,
                         lmaskcielocalcurve, localmaskcieutili,
-                        cielocalcurve, localcieutili, 
-                        cielocalcurve2, localcieutili2, 
-                        jzlocalcurve, localjzutili, 
-                        czlocalcurve, localczutili, 
-                        czjzlocalcurve, localczjzutili, 
-                        
+                        cielocalcurve, localcieutili,
+                        cielocalcurve2, localcieutili2,
+                        jzlocalcurve, localjzutili,
+                        czlocalcurve, localczutili,
+                        czjzlocalcurve, localczjzutili,
+
                         locccmasCurve, lcmasutili, locllmasCurve, llmasutili, lochhmasCurve, lhmasutili, lochhhmasCurve, lhhmasutili, locccmasexpCurve, lcmasexputili, locllmasexpCurve, llmasexputili, lochhmasexpCurve, lhmasexputili,
                         locccmasSHCurve, lcmasSHutili, locllmasSHCurve, llmasSHutili, lochhmasSHCurve, lhmasSHutili,
                         locccmasvibCurve, lcmasvibutili, locllmasvibCurve, llmasvibutili, lochhmasvibCurve, lhmasvibutili,
@@ -1229,7 +1307,8 @@ private:
                         LHutili, HHutili, CHutili, HHutilijz, CHutilijz, LHutilijz, cclocalcurve, localcutili, rgblocalcurve, localrgbutili, localexutili, exlocalcurve, hltonecurveloc, shtonecurveloc, tonecurveloc, lightCurveloc,
                         huerefblu, chromarefblu, lumarefblu, huere, chromare, lumare, sobelre, lastsav, false, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                         minCD, maxCD, mini, maxi, Tmean, Tsigma, Tmin, Tmax,
-                        meantme, stdtme, meanretie, stdretie, fab
+                        meantme, stdtme, meanretie, stdretie, fab,
+                        highresi, nresi, highresi46, nresi46, Lhighresi, Lnresi, Lhighresi46, Lnresi46
 );
 
                 if (sp + 1u < params.locallab.spots.size()) {
@@ -1467,8 +1546,8 @@ private:
             bool proedge = WaveParams.expedge;
             bool profin = WaveParams.expfinal;
             bool proton = WaveParams.exptoning;
-            bool pronois = WaveParams.expnoise; 
-            
+            bool pronois = WaveParams.expnoise;
+
 /*
             if(WaveParams.showmask) {
                 WaveParams.showmask = false;
@@ -1495,7 +1574,7 @@ private:
                 WaveParams.expedge = false;
                 WaveParams.expfinal = false;
                 WaveParams.exptoning = false;
-                WaveParams.expnoise = false; 
+                WaveParams.expnoise = false;
             }
 
             ipf.ip_wavelet(labView, labView, 2, WaveParams, wavCLVCurve, wavdenoise, wavdenoiseh, wavblcurve, waOpacityCurveRG, waOpacityCurveSH, waOpacityCurveBY, waOpacityCurveW,  waOpacityCurveWL, wavclCurve, 1);
@@ -1507,7 +1586,7 @@ private:
                 WaveParams.expfinal = profin;
                 WaveParams.exptoning = proton;
                 WaveParams.expnoise = pronois;
-                
+
                 if (WaveParams.softrad > 0.f) {
                     array2D<float> ble(fw, fh);
                     array2D<float> guid(fw, fh);
@@ -1562,7 +1641,7 @@ private:
                         }
                 delete tmpImage;
                 }
-                
+
             }
 
             if ((WaveParams.ushamethod == "sharp" || WaveParams.ushamethod == "clari") && WaveParams.expclari && WaveParams.CLmethod != "all") {
@@ -1652,7 +1731,7 @@ private:
                     labView->b[x][y] = 0.f;
                 }
             }
-           
+
         }
 
         //Colorappearance and tone-mapping associated
@@ -1679,18 +1758,11 @@ private:
 
         if (params.colorappearance.enabled) {
             double adap;
-            int imgNum = 0;
 
-            if (imgsrc->getSensorType() == ST_BAYER) {
-                imgNum = params.raw.bayersensor.imageNum;
-            } else if (imgsrc->getSensorType() == ST_FUJI_XTRANS) {
-                //imgNum = params.raw.xtranssensor.imageNum;
-            }
-
-            float fnum = imgsrc->getMetaData()->getFNumber(imgNum);          // F number
-            float fiso = imgsrc->getMetaData()->getISOSpeed(imgNum) ;        // ISO
-            float fspeed = imgsrc->getMetaData()->getShutterSpeed(imgNum) ;  //speed
-            float fcomp = imgsrc->getMetaData()->getExpComp(imgNum);         //compensation + -
+            const float fnum = imgsrc->getMetaData()->getFNumber();         // F number
+            const float fiso = imgsrc->getMetaData()->getISOSpeed() ;       // ISO
+            const float fspeed = imgsrc->getMetaData()->getShutterSpeed() ; // Speed
+            const float fcomp = imgsrc->getMetaData()->getExpComp();        // Compensation + -
 
             if (fnum < 0.3f || fiso < 5.f || fspeed < 0.00001f) {
                 adap = 2000.;
@@ -1823,21 +1895,22 @@ private:
             readyImg = tempImage;
         }
 
+        Exiv2Metadata info(imgsrc->getFileName());
         switch (params.metadata.mode) {
-            case MetaDataParams::TUNNEL:
-                // Sending back the whole first root, which won't necessarily be the selected frame number
-                // and may contain subframe depending on initial raw's hierarchy
-                readyImg->setMetadata(initialImage->getMetaData()->getRootExifData());
-                break;
-
-            case MetaDataParams::EDIT:
-                // ask for the correct frame number, but may contain subframe depending on initial raw's hierarchy
-                readyImg->setMetadata(initialImage->getMetaData()->getBestExifData(imgsrc, &params.raw), params.exif, params.iptc);
-                break;
-
-            default: // case MetaDataParams::STRIP
-                // nothing to do
-                break;
+        case MetaDataParams::TUNNEL:
+            readyImg->setMetadata(std::move(info));
+            break;
+        case MetaDataParams::EDIT:
+            info.setExif(params.metadata.exif);
+            info.setIptc(params.metadata.iptc);
+            if (!(params.metadata.exifKeys.size() == 1 && params.metadata.exifKeys[0] == "*")) {
+                info.setExifKeys(&(params.metadata.exifKeys));
+            }
+            readyImg->setMetadata(std::move(info));
+            break;
+        default: // case MetaDataParams::STRIP
+            // nothing to do
+            break;
         }
 
 
@@ -1859,11 +1932,11 @@ private:
                 }
 
                 ProfileContent pc = ICCStore::getInstance()->getContent(params.icm.outputProfile);
-                readyImg->setOutputProfile(pc.getData().c_str(), pc.getData().size());
+                readyImg->setOutputProfile(pc.getData());
             }
         } else {
             // No ICM
-            readyImg->setOutputProfile(nullptr, 0);
+            readyImg->setOutputProfile({});
         }
 
 //    t2.set();
@@ -2112,7 +2185,6 @@ IImagefloat* processImage(ProcessingJob* pjob, int& errorCode, ProgressListener*
 
 void batchProcessingThread(ProcessingJob* job, BatchProcessingListener* bpl)
 {
-
     ProcessingJob* currentJob = job;
 
     while (currentJob) {
