@@ -34,7 +34,7 @@ bool BatchQueueEntry::iconsLoaded(false);
 Glib::RefPtr<Gdk::Pixbuf> BatchQueueEntry::savedAsIcon;
 
 BatchQueueEntry::BatchQueueEntry (rtengine::ProcessingJob* pjob, const rtengine::procparams::ProcParams& pparams, Glib::ustring fname, int prevw, int prevh, Thumbnail* thm, bool overwrite) :
-    ThumbBrowserEntryBase(fname),
+    ThumbBrowserEntryBase(fname, thm),
     opreview(nullptr),
     origpw(prevw),
     origph(prevh),
@@ -50,7 +50,7 @@ BatchQueueEntry::BatchQueueEntry (rtengine::ProcessingJob* pjob, const rtengine:
 
     thumbnail = thm;
 
-#if 1 //ndef WIN32
+#if 1 //ndef _WIN32
     // The BatchQueueEntryIdleHelper tracks if an entry has been deleted while it was sitting waiting for "idle"
     bqih = new BatchQueueEntryIdleHelper;
     bqih->bqentry = this;
@@ -201,6 +201,9 @@ std::tuple<Glib::ustring, bool> BatchQueueEntry::getToolTip (int x, int y) const
                 if (saveFormat.tiffUncompressed) {
                     tooltip += Glib::ustring::compose("\n%1", M("SAVEDLG_TIFFUNCOMPRESSED"));
                 }
+                if (saveFormat.bigTiff) {
+                    tooltip += Glib::ustring::compose("\n%1", M("SAVEDLG_BIGTIFF"));
+                }
             }
         }
     }
