@@ -16,15 +16,14 @@
  *  You should have received a copy of the GNU General Public License
  *  along with RawTherapee.  If not, see <https://www.gnu.org/licenses/>.
  */
-#ifndef _BROWSERFILTER_
-#define _BROWSERFILTER_
+#pragma once
+
+#include <vector>
 
 #include "exiffiltersettings.h"
-#include <glibmm.h>
 
 class BrowserFilter
 {
-
 public:
     bool showRanked[6];
     bool showCLabeled[6];
@@ -41,5 +40,3 @@ public:
 
     BrowserFilter ();
 };
-
-#endif
