@@ -14,6 +14,7 @@
 using namespace rtengine;
 using namespace rtengine::procparams;
 
+const Glib::ustring ColorToning::TOOL_NAME = "colortoning";
 
 namespace {
 
@@ -33,7 +34,7 @@ inline float round_ab(float v)
 } // namespace
 
 
-ColorToning::ColorToning () : FoldableToolPanel(this, "colortoning", M("TP_COLORTONING_LABEL"), false, true)
+ColorToning::ColorToning () : FoldableToolPanel(this, TOOL_NAME, M("TP_COLORTONING_LABEL"), false, true)
 {
     nextbw = 0;
     CurveListener::setMulti(true);
