@@ -24,6 +24,7 @@
 #include "options.h"
 #include "../rtengine/procparams.h"
 #include "rtimage.h"
+#include "eventmapper.h"
 
 using namespace rtengine;
 using namespace procparams;
@@ -55,6 +56,7 @@ ControlSpotPanel::ControlSpotPanel():
     qualityMethod_(Gtk::manage(new MyComboBoxText())),
     //complexMethod_(Gtk::manage(new MyComboBoxText())),
     wavMethod_(Gtk::manage(new MyComboBoxText())),
+    avoidgamutMethod_(Gtk::manage(new MyComboBoxText())),
 
     sensiexclu_(Gtk::manage(new Adjuster(M("TP_LOCALLAB_SENSIEXCLU"), 0, 100, 1, 12))),
     structexclu_(Gtk::manage(new Adjuster(M("TP_LOCALLAB_STRUCCOL"), 0, 100, 1, 0))),
@@ -76,15 +78,13 @@ ControlSpotPanel::ControlSpotPanel():
     balanh_(Gtk::manage(new Adjuster(M("TP_LOCALLAB_BALANH"), 0.2, 2.5, 0.1, 1.0, Gtk::manage(new RTImage("rawtherapee-logo-16.png")), Gtk::manage(new RTImage("circle-red-green-small.png"))))),
     colorde_(Gtk::manage(new Adjuster(M("TP_LOCALLAB_COLORDE"), -15, 15, 2, 5, Gtk::manage(new RTImage("circle-blue-yellow-small.png")), Gtk::manage(new RTImage("circle-gray-green-small.png"))))),
     colorscope_(Gtk::manage(new Adjuster(M("TP_LOCALLAB_COLORSCOPE"), 0., 100.0, 1., 30.))),
-    avoidrad_(Gtk::manage(new Adjuster(M("TP_LOCALLAB_AVOIDRAD"), 0., 30.0, 0.1, 0.7))),
+    avoidrad_(Gtk::manage(new Adjuster(M("TP_LOCALLAB_AVOIDRAD"), 0., 30.0, 0.1, 0.))),
     scopemask_(Gtk::manage(new Adjuster(M("TP_LOCALLAB_SCOPEMASK"), 0, 100, 1, 60))),
     denoichmask_(Gtk::manage(new Adjuster(M("TP_LOCALLAB_DENOIMASK"), 0., 100., 0.5, 0))),
     lumask_(Gtk::manage(new Adjuster(M("TP_LOCALLAB_LUMASK"), -50, 30, 1, 10, Gtk::manage(new RTImage("circle-yellow-small.png")), Gtk::manage(new RTImage("circle-gray-small.png")) ))),
 
     hishow_(Gtk::manage(new Gtk::CheckButton(M("TP_LOCALLAB_PREVSHOW")))),
     activ_(Gtk::manage(new Gtk::CheckButton(M("TP_LOCALLAB_ACTIVSPOT")))),
-    avoid_(Gtk::manage(new Gtk::CheckButton(M("TP_LOCALLAB_AVOID")))),
-    avoidmun_(Gtk::manage(new Gtk::CheckButton(M("TP_LOCALLAB_AVOIDMUN")))),
     blwh_(Gtk::manage(new Gtk::CheckButton(M("TP_LOCALLAB_BLWH")))),
     recurs_(Gtk::manage(new Gtk::CheckButton(M("TP_LOCALLAB_RECURS")))),
     laplac_(Gtk::manage(new Gtk::CheckButton(M("TP_LOCALLAB_LAPLACC")))),
@@ -100,6 +100,7 @@ ControlSpotPanel::ControlSpotPanel():
     preview_(Gtk::manage(new Gtk::ToggleButton(M("TP_LOCALLAB_PREVIEW")))),
     ctboxshape(Gtk::manage(new Gtk::Box())),
     ctboxshapemethod(Gtk::manage(new Gtk::Box())),
+    ctboxgamut(Gtk::manage(new Gtk::Box())),
 
     controlPanelListener(nullptr),
     lastObject_(-1),
@@ -111,6 +112,8 @@ ControlSpotPanel::ControlSpotPanel():
     excluFrame(Gtk::manage(new Gtk::Frame(M("TP_LOCALLAB_EXCLUF")))),
     maskPrevActive(false)
 {
+    auto m = ProcEventMapper::getInstance();
+    EvLocallabavoidgamutMethod = m->newEvent(AUTOEXP, "HISTORY_MSG_LOCAL_GAMUTMUNSEL");
     const bool showtooltip = options.showtooltip;
     pack_start(*hishow_);
 
@@ -397,23 +400,30 @@ ControlSpotPanel::ControlSpotPanel():
     activConn_  = activ_->signal_toggled().connect(
                      sigc::mem_fun(*this, &ControlSpotPanel::activChanged));
 
-    avoidConn_  = avoid_->signal_toggled().connect(
-                      sigc::mem_fun(*this, &ControlSpotPanel::avoidChanged));
-    avoidmunConn_  = avoidmun_->signal_toggled().connect(
-                      sigc::mem_fun(*this, &ControlSpotPanel::avoidmunChanged));
-    
-    Gtk::Frame* const avFrame = Gtk::manage(new Gtk::Frame());
+    Gtk::Label* const labelgamut = Gtk::manage(new Gtk::Label(M("TP_LOCALLAB_AVOID") + ":"));
+    ctboxgamut->pack_start(*labelgamut, Gtk::PACK_SHRINK, 4);
+    avoidgamutMethod_->append(M("TP_LOCALLAB_GAMUTNON"));
+    avoidgamutMethod_->append(M("TP_LOCALLAB_GAMUTLABRELA"));
+    avoidgamutMethod_->append(M("TP_LOCALLAB_GAMUTXYZABSO"));
+    avoidgamutMethod_->append(M("TP_LOCALLAB_GAMUTXYZRELA"));
+    avoidgamutMethod_->append(M("TP_LOCALLAB_GAMUTMUNSELL"));
+    avoidgamutMethod_->set_active(2);
+    avoidgamutconn_ = avoidgamutMethod_->signal_changed().connect(
+                     sigc::mem_fun(
+                         *this, &ControlSpotPanel::avoidgamutMethodChanged));
+    ctboxgamut->pack_start(*avoidgamutMethod_);
+    if (showtooltip) {
+        ctboxgamut->set_tooltip_text(M("TP_LOCALLAB_AVOIDCOLORSHIFT_TOOLTIP"));
+    }
+
+ Gtk::Frame* const avFrame = Gtk::manage(new Gtk::Frame());
     ToolParamBlock* const avbox = Gtk::manage(new ToolParamBlock());
     avFrame->set_label_align(0.025, 0.5);
-    avFrame->set_label_widget(*avoid_);
+    avbox->pack_start(*ctboxgamut);
     avbox->pack_start(*avoidrad_);
-    avbox->pack_start(*avoidmun_);
     avFrame->add(*avbox);
     specCaseBox->pack_start(*avFrame);
 
-    if (showtooltip) {
-        avoidmun_->set_tooltip_text(M("TP_LOCALLAB_AVOIDMUN_TOOLTIP"));
-    }
 
     blwhConn_  = blwh_->signal_toggled().connect(
                      sigc::mem_fun(*this, &ControlSpotPanel::blwhChanged));
@@ -429,7 +439,6 @@ ControlSpotPanel::ControlSpotPanel():
 
     if (showtooltip) {
         recurs_->set_tooltip_text(M("TP_LOCALLAB_RECURS_TOOLTIP"));
-        avoid_->set_tooltip_text(M("TP_LOCALLAB_AVOIDCOLORSHIFT_TOOLTIP"));
     }
 
     specCaseBox->pack_start(*recurs_);
@@ -651,7 +660,7 @@ void ControlSpotPanel::on_button_delete()
     nbSpotChanged_ = true;
     selSpotChanged_ = true;
     eventType = SpotDeletion;
-    SpotRow* const delSpotRow = getSpot(selIndex);
+    const std::unique_ptr<SpotRow> delSpotRow = getSpot(selIndex);
     listener->panelChanged(EvLocallabSpotDeleted, delSpotRow->name);
 }
 
@@ -673,7 +682,7 @@ void ControlSpotPanel::on_button_duplicate()
     nbSpotChanged_ = true;
     selSpotChanged_ = true;
     eventType = SpotDuplication;
-    SpotRow* const duplSpotRow = getSpot(selIndex);
+    const std::unique_ptr<SpotRow> duplSpotRow = getSpot(selIndex);
     listener->panelChanged(EvLocallabSpotCreated, M("TP_LOCALLAB_EV_DUPL") + " "
                            + duplSpotRow->name);
 }
@@ -770,7 +779,7 @@ bool ControlSpotPanel::on_button_visibility(GdkEventButton* event)
 
             // Raise event
             visibilityChanged_ = true;
-            SpotRow* const spotRow = getSpot(getSelectedSpot());
+            const std::unique_ptr<SpotRow> spotRow = getSpot(getSelectedSpot());
 
             if (row[spots_.isvisible]) {
                 listener->panelChanged(EvLocallabSpotVisibility, M("TP_LOCALLAB_EV_VIS") + " (" + spotRow->name + ")");
@@ -854,8 +863,6 @@ void ControlSpotPanel::load_ControlSpot_param()
     avoidrad_->setValue((double)row[spots_.avoidrad]);
     hishow_->set_active(row[spots_.hishow]);
     activ_->set_active(row[spots_.activ]);
-    avoid_->set_active(row[spots_.avoid]);
-    avoidmun_->set_active(row[spots_.avoidmun]);
     blwh_->set_active(row[spots_.blwh]);
     recurs_->set_active(row[spots_.recurs]);
    // laplac_->set_active(row[spots_.laplac]);
@@ -868,6 +875,8 @@ void ControlSpotPanel::load_ControlSpot_param()
     //savrest_->set_active(row[spots_.savrest]);
     //complexMethod_->set_active(row[spots_.complexMethod]);
     wavMethod_->set_active(row[spots_.wavMethod]);
+	avoidgamutMethod_->set_active(row[spots_.avoidgamutMethod]);
+
 }
 
 void ControlSpotPanel::controlspotChanged()
@@ -887,7 +896,7 @@ void ControlSpotPanel::controlspotChanged()
 
     selSpotChanged_ = true;
     eventType = SpotSelection;
-    SpotRow* const spotRow = getSpot(selIndex);
+    const std::unique_ptr<SpotRow> spotRow = getSpot(selIndex);
 
     // Image area shall be regenerated if mask or deltaE preview was active when switching spot
     if (maskPrevActive || preview_->get_active()) {
@@ -1055,6 +1064,34 @@ void ControlSpotPanel::spotMethodChanged()
     }
 }
 
+void ControlSpotPanel::avoidgamutMethodChanged()
+{
+
+    // Get selected control spot
+    const auto s = treeview_->get_selection();
+
+    if (!s->count_selected_rows()) {
+        return;
+    }
+    const int meth = avoidgamutMethod_->get_active_row_number();
+    avoidrad_->show();
+
+    if(meth == 2 || meth == 3 || meth == 4) {
+        avoidrad_->hide();
+    }
+
+    const auto iter = s->get_selected();
+    Gtk::TreeModel::Row row = *iter;
+
+    row[spots_.avoidgamutMethod] = avoidgamutMethod_->get_active_row_number();
+
+    // Raise event
+    if (listener) {
+        listener->panelChanged(EvLocallabavoidgamutMethod, avoidgamutMethod_->get_active_text());
+    }
+
+}
+
 void ControlSpotPanel::shapeMethodChanged()
 {
     // printf("shapeMethodChanged\n");
@@ -1217,6 +1254,7 @@ void ControlSpotPanel::updateParamVisibility()
 
     // Update Control Spot GUI according to shapeMethod_ combobox state (to be compliant with shapeMethodChanged function)
     const int method = shapeMethod_->get_active_row_number();
+    const int meth = avoidgamutMethod_->get_active_row_number();
 
     if (!batchMode) {
         if (method == 1 || method == 3) { // Symmetrical cases
@@ -1260,6 +1298,12 @@ void ControlSpotPanel::updateParamVisibility()
         centerY_->show();
     }
 
+    if(meth == 1) {
+        avoidrad_->show();
+    } else {
+        avoidrad_->hide();
+}
+
     // Update Control Spot GUI according to spotMethod_ combobox state (to be compliant with spotMethodChanged function)
     if (multiImage && spotMethod_->get_active_text() == M("GENERAL_UNCHANGED")) {
         excluFrame->show();
@@ -1588,57 +1632,6 @@ void ControlSpotPanel::hishowChanged()
 }
 
 
-
-void ControlSpotPanel::avoidChanged()
-{
-    // printf("avoidChanged\n");
-
-    // Get selected control spot
-    const auto s = treeview_->get_selection();
-
-    if (!s->count_selected_rows()) {
-        return;
-    }
-
-    const auto iter = s->get_selected();
-    Gtk::TreeModel::Row row = *iter;
-    row[spots_.avoid] = avoid_->get_active();
-
-    // Raise event
-    if (listener) {
-        if (avoid_->get_active()) {
-            listener->panelChanged(Evlocallabavoid, M("GENERAL_ENABLED"));
-        } else {
-            listener->panelChanged(Evlocallabavoid, M("GENERAL_DISABLED"));
-        }
-    }
-}
-
-void ControlSpotPanel::avoidmunChanged()
-{
-    // printf("avoidmunChanged\n");
-
-    // Get selected control spot
-    const auto s = treeview_->get_selection();
-
-    if (!s->count_selected_rows()) {
-        return;
-    }
-
-    const auto iter = s->get_selected();
-    Gtk::TreeModel::Row row = *iter;
-    row[spots_.avoidmun] = avoidmun_->get_active();
-
-    // Raise event
-    if (listener) {
-        if (avoidmun_->get_active()) {
-            listener->panelChanged(EvLocallabSpotavoidmun, M("GENERAL_ENABLED"));
-        } else {
-            listener->panelChanged(EvLocallabSpotavoidmun, M("GENERAL_DISABLED"));
-        }
-    }
-}
-
 void ControlSpotPanel::activChanged()
 {
     // printf("activChanged\n");
@@ -1859,8 +1852,6 @@ void ControlSpotPanel::disableParamlistener(bool cond)
     avoidrad_->block(cond);
     hishowconn_.block(cond);
     activConn_.block(cond);
-    avoidConn_.block(cond);
-    avoidmunConn_.block(cond);
     blwhConn_.block(cond);
     recursConn_.block(cond);
     laplacConn_.block(cond);
@@ -1872,6 +1863,8 @@ void ControlSpotPanel::disableParamlistener(bool cond)
     //savrestConn_.block(cond);
     //complexMethodconn_.block(cond);
     wavMethodconn_.block(cond);
+	avoidgamutconn_.block(cond);
+
 }
 
 void ControlSpotPanel::setParamEditable(bool cond)
@@ -1906,8 +1899,6 @@ void ControlSpotPanel::setParamEditable(bool cond)
     avoidrad_->set_sensitive(cond);
     hishow_->set_sensitive(cond);
     activ_->set_sensitive(cond);
-    avoid_->set_sensitive(cond);
-    avoidmun_->set_sensitive(cond);
     blwh_->set_sensitive(cond);
     recurs_->set_sensitive(cond);
     laplac_->set_sensitive(cond);
@@ -1920,6 +1911,7 @@ void ControlSpotPanel::setParamEditable(bool cond)
     //complexMethod_->set_sensitive(cond);
     wavMethod_->set_sensitive(cond);
     preview_->set_sensitive(cond);
+    avoidgamutMethod_->set_sensitive(cond);
 
     if (!cond) {
         // Reset complex parameters visibility to default state
@@ -1985,8 +1977,8 @@ void ControlSpotPanel::addControlSpotCurve(Gtk::TreeModel::Row& row)
     shape_ellipse = new Ellipse();
     shape_ellipse->datum = Geometry::IMAGE;
     shape_ellipse->radiusInImageSpace = true;
-    Rectangle* shape_rectangle;
-    shape_rectangle = new Rectangle();
+    EditRectangle* shape_rectangle;
+    shape_rectangle = new EditRectangle();
     shape_rectangle->datum = Geometry::IMAGE;
     EditSubscriber::visibleGeometry.push_back(centerCircle); // (curveid - 1) * 7
     EditSubscriber::visibleGeometry.push_back(shape_ellipse); // (curveid - 1) * 7 + 1
@@ -2020,7 +2012,7 @@ void ControlSpotPanel::addControlSpotCurve(Gtk::TreeModel::Row& row)
     shape_ellipse = new Ellipse();
     shape_ellipse->datum = Geometry::IMAGE;
     shape_ellipse->radiusInImageSpace = true;
-    shape_rectangle = new Rectangle();
+    shape_rectangle = new EditRectangle();
     shape_rectangle->datum = Geometry::IMAGE;
     EditSubscriber::mouseOverGeometry.push_back(centerCircle);  // (curveid - 1) * 7
     EditSubscriber::mouseOverGeometry.push_back(shape_ellipse);  // (curveid - 1) * 7 + 1
@@ -2090,7 +2082,7 @@ void ControlSpotPanel::updateControlSpotCurve(const Gtk::TreeModel::Row& row)
     };
 
     const auto updateRectangle = [&](Geometry * geometry) {
-        const auto rectangle = static_cast<Rectangle*>(geometry);
+        const auto rectangle = static_cast<EditRectangle*>(geometry);
         rectangle->bottomRight.x = origin.x + decayX;
         rectangle->bottomRight.y = origin.y + decayY;
         rectangle->topLeft.x = origin.x - decayXL;
@@ -2543,13 +2535,16 @@ int ControlSpotPanel::getEventType()
     return tmp;
 }
 
-ControlSpotPanel::SpotRow* ControlSpotPanel::getSpot(const int index)
+std::unique_ptr<ControlSpotPanel::SpotRow> ControlSpotPanel::getSpot(const int index)
 {
+    // TODO: Return an std::optional<ControlSpotPanel::SpotRow> after upgrading
+    // to C++17.
+
     // printf("getSpot: %d\n", index);
 
     MyMutex::MyLock lock(mTreeview);
 
-    SpotRow* r = new SpotRow();
+    std::unique_ptr<SpotRow> r(new SpotRow());
 
     int i = -1;
 
@@ -2592,8 +2587,6 @@ ControlSpotPanel::SpotRow* ControlSpotPanel::getSpot(const int index)
             r->lumask = row[spots_.lumask];
             r->hishow = row[spots_.hishow];
             r->activ = row[spots_.activ];
-            r->avoid = row[spots_.avoid];
-            r->avoidmun = row[spots_.avoidmun];
             r->blwh = row[spots_.blwh];
             r->recurs = row[spots_.recurs];
             r->laplac = row[spots_.laplac];
@@ -2601,6 +2594,7 @@ ControlSpotPanel::SpotRow* ControlSpotPanel::getSpot(const int index)
             r->shortc = row[spots_.shortc];
             //r->savrest = row[spots_.savrest];
             r->wavMethod = row[spots_.wavMethod];
+            r->avoidgamutMethod = row[spots_.avoidgamutMethod];
 
             return r;
         }
@@ -2685,59 +2679,58 @@ void ControlSpotPanel::resetDeltaEPreview()
     previewConn_.block(false);
 }
 
-void ControlSpotPanel::addControlSpot(SpotRow* newSpot)
+void ControlSpotPanel::addControlSpot(const SpotRow &newSpot)
 {
-    // printf("addControlSpot: %d\n", newSpot->name);
+    // printf("addControlSpot: %d\n", newSpot.name);
 
     MyMutex::MyLock lock(mTreeview);
 
     disableParamlistener(true);
     Gtk::TreeModel::Row row = *(treemodel_->append());
     row[spots_.mouseover] = false;
-    row[spots_.name] = newSpot->name;
-    row[spots_.isvisible] = newSpot->isvisible;
+    row[spots_.name] = newSpot.name;
+    row[spots_.isvisible] = newSpot.isvisible;
     row[spots_.curveid] = 0; // No associated curve
-    row[spots_.prevMethod] = newSpot->prevMethod;
-    row[spots_.shape] = newSpot->shape;
-    row[spots_.spotMethod] = newSpot->spotMethod;
-    row[spots_.sensiexclu] = newSpot->sensiexclu;
-    row[spots_.structexclu] = newSpot->structexclu;
-    row[spots_.shapeMethod] = newSpot->shapeMethod;
-    row[spots_.locX] = newSpot->locX;
-    row[spots_.locXL] = newSpot->locXL;
-    row[spots_.locY] = newSpot->locY;
-    row[spots_.locYT] = newSpot->locYT;
-    row[spots_.centerX] = newSpot->centerX;
-    row[spots_.centerY] = newSpot->centerY;
-    row[spots_.circrad] = newSpot->circrad;
-    row[spots_.qualityMethod] = newSpot->qualityMethod;
-    row[spots_.transit] = newSpot->transit;
-    row[spots_.transitweak] = newSpot->transitweak;
-    row[spots_.transitgrad] = newSpot->transitgrad;
-    row[spots_.feather] = newSpot->feather;
-    row[spots_.struc] = newSpot->struc;
-    row[spots_.thresh] = newSpot->thresh;
-    row[spots_.iter] = newSpot->iter;
-    row[spots_.balan] = newSpot->balan;
-    row[spots_.balanh] = newSpot->balanh;
-    row[spots_.colorde] = newSpot->colorde;
-    row[spots_.colorscope] = newSpot->colorscope;
-    row[spots_.avoidrad] = newSpot->avoidrad;
-    row[spots_.hishow] = newSpot->hishow;
-    row[spots_.activ] = newSpot->activ;
-    row[spots_.avoid] = newSpot->avoid;
-    row[spots_.avoidmun] = newSpot->avoidmun;
-    row[spots_.blwh] = newSpot->blwh;
-    row[spots_.recurs] = newSpot->recurs;
-    row[spots_.laplac] = newSpot->laplac;
-    row[spots_.deltae] = newSpot->deltae;
-    row[spots_.scopemask] = newSpot->scopemask;
-    row[spots_.denoichmask] = newSpot->denoichmask;
-    row[spots_.shortc] = newSpot->shortc;
-    row[spots_.lumask] = newSpot->lumask;
-    //row[spots_.savrest] = newSpot->savrest;
-    row[spots_.complexMethod] = newSpot->complexMethod;
-    row[spots_.wavMethod] = newSpot->wavMethod;
+    row[spots_.prevMethod] = newSpot.prevMethod;
+    row[spots_.shape] = newSpot.shape;
+    row[spots_.spotMethod] = newSpot.spotMethod;
+    row[spots_.sensiexclu] = newSpot.sensiexclu;
+    row[spots_.structexclu] = newSpot.structexclu;
+    row[spots_.shapeMethod] = newSpot.shapeMethod;
+    row[spots_.locX] = newSpot.locX;
+    row[spots_.locXL] = newSpot.locXL;
+    row[spots_.locY] = newSpot.locY;
+    row[spots_.locYT] = newSpot.locYT;
+    row[spots_.centerX] = newSpot.centerX;
+    row[spots_.centerY] = newSpot.centerY;
+    row[spots_.circrad] = newSpot.circrad;
+    row[spots_.qualityMethod] = newSpot.qualityMethod;
+    row[spots_.transit] = newSpot.transit;
+    row[spots_.transitweak] = newSpot.transitweak;
+    row[spots_.transitgrad] = newSpot.transitgrad;
+    row[spots_.feather] = newSpot.feather;
+    row[spots_.struc] = newSpot.struc;
+    row[spots_.thresh] = newSpot.thresh;
+    row[spots_.iter] = newSpot.iter;
+    row[spots_.balan] = newSpot.balan;
+    row[spots_.balanh] = newSpot.balanh;
+    row[spots_.colorde] = newSpot.colorde;
+    row[spots_.colorscope] = newSpot.colorscope;
+    row[spots_.avoidrad] = newSpot.avoidrad;
+    row[spots_.hishow] = newSpot.hishow;
+    row[spots_.activ] = newSpot.activ;
+    row[spots_.blwh] = newSpot.blwh;
+    row[spots_.recurs] = newSpot.recurs;
+    row[spots_.laplac] = newSpot.laplac;
+    row[spots_.deltae] = newSpot.deltae;
+    row[spots_.scopemask] = newSpot.scopemask;
+    row[spots_.denoichmask] = newSpot.denoichmask;
+    row[spots_.shortc] = newSpot.shortc;
+    row[spots_.lumask] = newSpot.lumask;
+    //row[spots_.savrest] = newSpot.savrest;
+    row[spots_.complexMethod] = newSpot.complexMethod;
+    row[spots_.wavMethod] = newSpot.wavMethod;
+    row[spots_.avoidgamutMethod] = newSpot.avoidgamutMethod;
     updateParamVisibility();
     disableParamlistener(false);
 
@@ -2845,8 +2838,6 @@ ControlSpotPanel::ControlSpots::ControlSpots()
     add(avoidrad);
     add(hishow);
     add(activ);
-    add(avoid);
-    add(avoidmun);
     add(blwh);
     add(recurs);
     add(laplac);
@@ -2858,6 +2849,7 @@ ControlSpotPanel::ControlSpots::ControlSpots()
     //add(savrest);
     add(complexMethod);
     add(wavMethod);
+	add(avoidgamutMethod);
 }
 
 //-----------------------------------------------------------------------------
