@@ -24,6 +24,7 @@
 #include "guiutils.h"
 
 class EditorPanel;
+struct ExternalEditor;
 class RTWindow;
 
 class EditWindow :
@@ -39,6 +40,8 @@ private:
     std::set<Glib::ustring> filesEdited;
     std::map<Glib::ustring, EditorPanel*> epanels;
 
+    sigc::signal<void> externalEditorChangedSignal;
+
     bool isFullscreen;
     bool isClosed;
     bool isMinimized;
@@ -64,6 +67,9 @@ public:
     bool selectEditorPanel(const std::string &name);
     bool closeOpenEditors();
     bool isProcessing();
+    void updateExternalEditorWidget(int selectedIndex, const std::vector<ExternalEditor> &editors);
+    void updateToolPanelToolLocations(
+        const std::vector<Glib::ustring> &favorites, bool cloneFavoriteTools);
 
     void toFront();
     bool keyPressed (GdkEventKey* event);
