@@ -167,6 +167,41 @@ FileBrowser::FileBrowser () :
     pmenu->attach (*Gtk::manage(selall = new Gtk::MenuItem (M("FILEBROWSER_POPUPSELECTALL"))), 0, 1, p, p + 1);
     p++;
 
+    /***********************
+     * sort
+     ***********************/
+    const std::array<std::string, 2> cnameSortOrders = {
+        M("SORT_ASCENDING"),
+        M("SORT_DESCENDING"),
+    };
+
+    const std::array<std::string, Options::SORT_METHOD_COUNT> cnameSortMethods = {
+        M("SORT_BY_NAME"),
+        M("SORT_BY_DATE"),
+        M("SORT_BY_EXIF"),
+        M("SORT_BY_RANK"),
+        M("SORT_BY_LABEL"),
+    };
+
+    pmenu->attach (*Gtk::manage(menuSort = new Gtk::MenuItem (M("FILEBROWSER_POPUPSORTBY"))), 0, 1, p, p + 1);
+    p++;
+    Gtk::Menu* submenuSort = Gtk::manage (new Gtk::Menu ());
+    Gtk::RadioButtonGroup sortOrderGroup, sortMethodGroup;
+    for (size_t i = 0; i < cnameSortOrders.size(); i++) {
+        submenuSort->attach (*Gtk::manage(sortOrder[i] = new Gtk::RadioMenuItem (sortOrderGroup, cnameSortOrders[i])), 0, 1, p, p + 1);
+        p++;
+        sortOrder[i]->set_active (i == options.sortDescending);
+    }
+    submenuSort->attach (*Gtk::manage(new Gtk::SeparatorMenuItem ()), 0, 1, p, p + 1);
+    p++;
+    for (size_t i = 0; i < cnameSortMethods.size(); i++) {
+        submenuSort->attach (*Gtk::manage(sortMethod[i] = new Gtk::RadioMenuItem (sortMethodGroup, cnameSortMethods[i])), 0, 1, p, p + 1);
+        p++;
+        sortMethod[i]->set_active (i == options.sortMethod);
+    }
+    submenuSort->show_all ();
+    menuSort->set_submenu (*submenuSort);
+
     /***********************
      * rank
      ***********************/
@@ -237,7 +272,7 @@ FileBrowser::FileBrowser () :
     /***********************
      * external programs
      * *********************/
-#if defined(WIN32)
+#if defined(_WIN32)
     Gtk::manage(miOpenDefaultViewer = new Gtk::MenuItem (M("FILEBROWSER_OPENDEFAULTVIEWER")));
 #endif
 
@@ -261,7 +296,7 @@ FileBrowser::FileBrowser () :
             p++;
             Gtk::Menu* submenuExtProg = Gtk::manage (new Gtk::Menu());
 
-#ifdef WIN32
+#ifdef _WIN32
             if (miOpenDefaultViewer) {
                 submenuExtProg->attach (*miOpenDefaultViewer, 0, 1, p, p + 1);
                 p++;
@@ -275,7 +310,7 @@ FileBrowser::FileBrowser () :
             submenuExtProg->show_all ();
             menuExtProg->set_submenu (*submenuExtProg);
         } else {
-#ifdef WIN32
+#ifdef _WIN32
             if (miOpenDefaultViewer) {
                 pmenu->attach (*miOpenDefaultViewer, 0, 1, p, p + 1);
                 p++;
@@ -427,6 +462,14 @@ FileBrowser::FileBrowser () :
         inspect->signal_activate().connect (sigc::bind(sigc::mem_fun(*this, &FileBrowser::menuItemActivated), inspect));
     }
 
+    for (int i = 0; i < 2; i++) {
+        sortOrder[i]->signal_activate().connect (sigc::bind(sigc::mem_fun(*this, &FileBrowser::menuItemActivated), sortOrder[i]));
+    }
+
+    for (int i = 0; i < Options::SORT_METHOD_COUNT; i++) {
+        sortMethod[i]->signal_activate().connect (sigc::bind(sigc::mem_fun(*this, &FileBrowser::menuItemActivated), sortMethod[i]));
+    }
+
     for (int i = 0; i < 6; i++) {
         rank[i]->signal_activate().connect (sigc::bind(sigc::mem_fun(*this, &FileBrowser::menuItemActivated), rank[i]));
     }
@@ -439,7 +482,7 @@ FileBrowser::FileBrowser () :
         amiExtProg[i]->signal_activate().connect (sigc::bind(sigc::mem_fun(*this, &FileBrowser::menuItemActivated), amiExtProg[i]));
     }
 
-#ifdef WIN32
+#ifdef _WIN32
     if (miOpenDefaultViewer) {
         miOpenDefaultViewer->signal_activate().connect (sigc::bind(sigc::mem_fun(*this, &FileBrowser::menuItemActivated), miOpenDefaultViewer));
     }
@@ -610,27 +653,7 @@ void FileBrowser::addEntry_ (FileBrowserEntry* entry)
     entry->getThumbButtonSet()->setButtonListener(this);
     entry->resize(getThumbnailHeight());
     entry->filtered = !checkFilter(entry);
-
-    // find place in abc order
-    {
-        MYWRITERLOCK(l, entryRW);
-
-        fd.insert(
-            std::lower_bound(
-                fd.begin(),
-                fd.end(),
-                entry,
-                [](const ThumbBrowserEntryBase* a, const ThumbBrowserEntryBase* b)
-                {
-                    return *a < *b;
-                }
-            ),
-            entry
-        );
-
-        initEntry(entry);
-    }
-    redraw(entry);
+    insertEntry(entry);
 }
 
 FileBrowserEntry* FileBrowser::delEntry (const Glib::ustring& fname)
@@ -724,6 +747,18 @@ void FileBrowser::menuItemActivated (Gtk::MenuItem* m)
         return;
     }
 
+    for (int i = 0; i < 2; i++)
+        if (m == sortOrder[i]) {
+            sortOrderRequested (i);
+            return;
+        }
+
+    for (int i = 0; i < Options::SORT_METHOD_COUNT; i++)
+        if (m == sortMethod[i]) {
+            sortMethodRequested (i);
+            return;
+        }
+
     for (int i = 0; i < 6; i++)
         if (m == rank[i]) {
             rankingRequested (mselected, i);
@@ -1011,7 +1046,7 @@ void FileBrowser::menuItemActivated (Gtk::MenuItem* m)
         tbl->clearFromCacheRequested (mselected, true);
 
         //queue_draw ();
-#ifdef WIN32
+#ifdef _WIN32
     } else if (miOpenDefaultViewer && m == miOpenDefaultViewer) {
         openDefaultViewer(1);
 #endif
@@ -1122,7 +1157,7 @@ void FileBrowser::partPasteProfile ()
     }
 }
 
-#ifdef WIN32
+#ifdef _WIN32
 void FileBrowser::openDefaultViewer (int destination)
 {
     bool success = true;
@@ -1219,7 +1254,7 @@ bool FileBrowser::keyPressed (GdkEventKey* event)
         }
 
         openRequested(mselected);
-#ifdef WIN32
+#ifdef _WIN32
     } else if (event->keyval == GDK_KEY_F5) {
         int dest = 1;
 
@@ -1241,7 +1276,7 @@ bool FileBrowser::keyPressed (GdkEventKey* event)
     }
 
 #ifdef __WIN32__
-    else if (shift && !ctrl && !alt && !altgr) { // rank
+    else if (!shift && !ctrl && !alt && !altgr) { // rank
         switch(event->hardware_keycode) {
         case 0x30:  // 0-key
             requestRanking (0);
@@ -1296,7 +1331,7 @@ bool FileBrowser::keyPressed (GdkEventKey* event)
     }
 
 #else
-    else if (shift && !ctrl && !alt) { // rank
+    else if (!shift && !ctrl && !alt) { // rank
         switch(event->hardware_keycode) {
         case 0x13:
             requestRanking (0);
@@ -1632,6 +1667,18 @@ void FileBrowser::fromTrashRequested (std::vector<FileBrowserEntry*> tbe)
     applyFilter (filter);
 }
 
+void FileBrowser::sortMethodRequested (int method)
+{
+    options.sortMethod = Options::SortMethod(method);
+    resort ();
+}
+
+void FileBrowser::sortOrderRequested (int order)
+{
+    options.sortDescending = !!order;
+    resort ();
+}
+
 void FileBrowser::rankingRequested (std::vector<FileBrowserEntry*> tbe, int rank)
 {
 
