@@ -556,6 +556,13 @@ void FlatCurveEditorSubGroup::restoreDisplayedHistogram()
 
 }
 
+void FlatCurveEditorSubGroup::restoreLocallabBackground()
+{
+    if (parent->displayedCurve) {
+        CPointsCurve->updateLocallabBackground(parent->displayedCurve->locallabRef);
+    }
+}
+
 void FlatCurveEditorSubGroup::storeCurveValues (CurveEditor* ce, const std::vector<double>& p)
 {
     if (!p.empty()) {
@@ -629,6 +636,13 @@ bool FlatCurveEditorSubGroup::curveReset(CurveEditor *ce)
     return true;
 }
 
+void FlatCurveEditorSubGroup::updateLocallabBackground(CurveEditor* ce)
+{
+    if (ce == parent->displayedCurve) {
+        CPointsCurve->updateLocallabBackground(ce->locallabRef);
+    }
+}
+
 /*void FlatCurveEditorSubGroup::updateBackgroundHistogram (CurveEditor* ce) {
     CurveEditor* fce = (CurveEditor*)ce;
     if (fce==displayedCurve) {
