@@ -29,6 +29,8 @@ using namespace procparams;
 
 extern Options options;
 
+const Glib::ustring Locallab::TOOL_NAME = "locallab";
+
 /* ==== LocallabToolList ==== */
 LocallabToolList::LocallabToolList():
     // Tool list GUI elements
@@ -142,7 +144,7 @@ void LocallabToolList::toolRowSelected()
 
 /* ==== Locallab ==== */
 Locallab::Locallab():
-    FoldableToolPanel(this, "locallab", M("TP_LOCALLAB_LABEL"), false, true),
+    FoldableToolPanel(this, TOOL_NAME, M("TP_LOCALLAB_LABEL"), false, true),
 
     // Spot control panel widget
     expsettings(Gtk::manage(new ControlSpotPanel())),
@@ -239,103 +241,113 @@ void Locallab::read(const rtengine::procparams::ProcParams* pp, const ParamsEdit
 
     // TODO Manage it with read function in controlspotpanel.cc
     // Add existent spots based on pp
-    ControlSpotPanel::SpotRow* const r = new ControlSpotPanel::SpotRow();
+    ControlSpotPanel::SpotRow r;
 
     for (int i = 0; i < (int)pp->locallab.spots.size(); i++) {
-        r->name = pp->locallab.spots.at(i).name;
-        r->isvisible = pp->locallab.spots.at(i).isvisible;
+        r.name = pp->locallab.spots.at(i).name;
+        r.isvisible = pp->locallab.spots.at(i).isvisible;
 
         if (pp->locallab.spots.at(i).shape == "ELI") {
-            r->shape = 0;
+            r.shape = 0;
         } else {
-            r->shape = 1;
+            r.shape = 1;
         }
 
         if (pp->locallab.spots.at(i).prevMethod == "hide") {
-            r->prevMethod = 0;
+            r.prevMethod = 0;
         } else {
-            r->prevMethod = 1;
+            r.prevMethod = 1;
         }
 
         if (pp->locallab.spots.at(i).spotMethod == "norm") {
-            r->spotMethod = 0;
+            r.spotMethod = 0;
         } else if(pp->locallab.spots.at(i).spotMethod == "exc"){
-            r->spotMethod = 1;
+            r.spotMethod = 1;
         } else if (pp->locallab.spots.at(i).spotMethod == "full"){
-            r->spotMethod = 2;
+            r.spotMethod = 2;
         }
 
-        r->sensiexclu = pp->locallab.spots.at(i).sensiexclu;
-        r->structexclu = pp->locallab.spots.at(i).structexclu;
+        r.sensiexclu = pp->locallab.spots.at(i).sensiexclu;
+        r.structexclu = pp->locallab.spots.at(i).structexclu;
 
         if (pp->locallab.spots.at(i).shapeMethod == "IND") {
-            r->shapeMethod = 0;
+            r.shapeMethod = 0;
         } else if (pp->locallab.spots.at(i).shapeMethod == "SYM") {
-            r->shapeMethod = 1;
+            r.shapeMethod = 1;
         } else if (pp->locallab.spots.at(i).shapeMethod == "INDSL") {
-            r->shapeMethod = 2;
+            r.shapeMethod = 2;
         } else {
-            r->shapeMethod = 3;
+            r.shapeMethod = 3;
         }
+		
+        if (pp->locallab.spots.at(i).avoidgamutMethod == "NONE") {
+            r.avoidgamutMethod = 0;
+        } else if (pp->locallab.spots.at(i).avoidgamutMethod == "LAB") {
+            r.avoidgamutMethod = 1;
+        } else if (pp->locallab.spots.at(i).avoidgamutMethod == "XYZ") {
+            r.avoidgamutMethod= 2;
+        } else if (pp->locallab.spots.at(i).avoidgamutMethod == "XYZREL") {
+            r.avoidgamutMethod= 3;
+        } else if (pp->locallab.spots.at(i).avoidgamutMethod == "MUNS") {
+            r.avoidgamutMethod= 4;
+        } 
 
-        r->locX = pp->locallab.spots.at(i).loc.at(0);
-        r->locXL = pp->locallab.spots.at(i).loc.at(1);
-        r->locY = pp->locallab.spots.at(i).loc.at(2);
-        r->locYT = pp->locallab.spots.at(i).loc.at(3);
-        r->centerX = pp->locallab.spots.at(i).centerX;
-        r->centerY = pp->locallab.spots.at(i).centerY;
-        r->circrad = pp->locallab.spots.at(i).circrad;
+        r.locX = pp->locallab.spots.at(i).loc.at(0);
+        r.locXL = pp->locallab.spots.at(i).loc.at(1);
+        r.locY = pp->locallab.spots.at(i).loc.at(2);
+        r.locYT = pp->locallab.spots.at(i).loc.at(3);
+        r.centerX = pp->locallab.spots.at(i).centerX;
+        r.centerY = pp->locallab.spots.at(i).centerY;
+        r.circrad = pp->locallab.spots.at(i).circrad;
 
         if (pp->locallab.spots.at(i).qualityMethod == "enh") {
-            r->qualityMethod = 0;
+            r.qualityMethod = 0;
         } else {
-            r->qualityMethod = 1;
+            r.qualityMethod = 1;
         }
 
-        r->transit = pp->locallab.spots.at(i).transit;
-        r->transitweak = pp->locallab.spots.at(i).transitweak;
-        r->transitgrad = pp->locallab.spots.at(i).transitgrad;
-        r->feather = pp->locallab.spots.at(i).feather;
-        r->struc = pp->locallab.spots.at(i).struc;
-        r->thresh = pp->locallab.spots.at(i).thresh;
-        r->iter = pp->locallab.spots.at(i).iter;
-        r->balan = pp->locallab.spots.at(i).balan;
-        r->balanh = pp->locallab.spots.at(i).balanh;
-        r->colorde = pp->locallab.spots.at(i).colorde;
-        r->colorscope = pp->locallab.spots.at(i).colorscope;
-        r->avoidrad = pp->locallab.spots.at(i).avoidrad;
-        r->hishow = pp->locallab.spots.at(i).hishow;
-        r->activ = pp->locallab.spots.at(i).activ;
-        r->avoid = pp->locallab.spots.at(i).avoid;
-        r->avoidmun = pp->locallab.spots.at(i).avoidmun;
-        r->blwh = pp->locallab.spots.at(i).blwh;
-        r->recurs = pp->locallab.spots.at(i).recurs;
-        r->laplac = true; //pp->locallab.spots.at(i).laplac;
-        r->deltae = pp->locallab.spots.at(i).deltae;
-        r->scopemask = pp->locallab.spots.at(i).scopemask;
-        r->denoichmask = pp->locallab.spots.at(i).denoichmask;
-        r->shortc = pp->locallab.spots.at(i).shortc;
-        r->lumask = pp->locallab.spots.at(i).lumask;
-        //r->savrest = pp->locallab.spots.at(i).savrest;
+        r.transit = pp->locallab.spots.at(i).transit;
+        r.transitweak = pp->locallab.spots.at(i).transitweak;
+        r.transitgrad = pp->locallab.spots.at(i).transitgrad;
+        r.feather = pp->locallab.spots.at(i).feather;
+        r.struc = pp->locallab.spots.at(i).struc;
+        r.thresh = pp->locallab.spots.at(i).thresh;
+        r.iter = pp->locallab.spots.at(i).iter;
+        r.balan = pp->locallab.spots.at(i).balan;
+        r.balanh = pp->locallab.spots.at(i).balanh;
+        r.colorde = pp->locallab.spots.at(i).colorde;
+        r.colorscope = pp->locallab.spots.at(i).colorscope;
+        r.avoidrad = pp->locallab.spots.at(i).avoidrad;
+        r.hishow = pp->locallab.spots.at(i).hishow;
+        r.activ = pp->locallab.spots.at(i).activ;
+        r.blwh = pp->locallab.spots.at(i).blwh;
+        r.recurs = pp->locallab.spots.at(i).recurs;
+        r.laplac = true; //pp->locallab.spots.at(i).laplac;
+        r.deltae = pp->locallab.spots.at(i).deltae;
+        r.scopemask = pp->locallab.spots.at(i).scopemask;
+        r.denoichmask = pp->locallab.spots.at(i).denoichmask;
+        r.shortc = pp->locallab.spots.at(i).shortc;
+        r.lumask = pp->locallab.spots.at(i).lumask;
+        //r.savrest = pp->locallab.spots.at(i).savrest;
 
         if (pp->locallab.spots.at(i).complexMethod == "sim") {
-            r->complexMethod = 0;
+            r.complexMethod = 0;
         } else  if (pp->locallab.spots.at(i).complexMethod == "mod") {
-            r->complexMethod = 1;
+            r.complexMethod = 1;
         } else  if (pp->locallab.spots.at(i).complexMethod == "all") {
-            r->complexMethod = 2;
+            r.complexMethod = 2;
         }
 
         if (pp->locallab.spots.at(i).wavMethod == "D2") {
-            r->wavMethod = 0;
+            r.wavMethod = 0;
         } else  if (pp->locallab.spots.at(i).wavMethod == "D4") {
-            r->wavMethod = 1;
+            r.wavMethod = 1;
         } else  if (pp->locallab.spots.at(i).wavMethod == "D6") {
-            r->wavMethod = 2;
+            r.wavMethod = 2;
         } else  if (pp->locallab.spots.at(i).wavMethod == "D10") {
-            r->wavMethod = 3;
+            r.wavMethod = 3;
         } else  if (pp->locallab.spots.at(i).wavMethod == "D14") {
-            r->wavMethod = 4;
+            r.wavMethod = 4;
         }
 
         expsettings->addControlSpot(r);
@@ -389,7 +401,6 @@ void Locallab::write(rtengine::procparams::ProcParams* pp, ParamsEdited* pedited
 
     const int spotPanelEvent = expsettings->getEventType();
     int spotIndex;
-    ControlSpotPanel::SpotRow* r;
     rtengine::procparams::LocallabParams::LocallabSpot* newSpot;
 
     int imW, imH; // Size of image
@@ -400,45 +411,57 @@ void Locallab::write(rtengine::procparams::ProcParams* pp, ParamsEdited* pedited
     int toolNb;
 
     switch (spotPanelEvent) {
-        case (ControlSpotPanel::SpotCreation): // Spot creation event
+        case (ControlSpotPanel::SpotCreation): { // Spot creation event
             // Spot creation (default initialization)
             newSpot = new LocallabParams::LocallabSpot();
-            r = new ControlSpotPanel::SpotRow();
-            r->name = newSpot->name = M("TP_LOCALLAB_SPOTNAME");
-            r->isvisible = newSpot->isvisible;
+            ControlSpotPanel::SpotRow r;
+            r.name = newSpot->name = M("TP_LOCALLAB_SPOTNAME");
+            r.isvisible = newSpot->isvisible;
 
             if (newSpot->shape == "ELI") {
-                r->shape = 0;
+                r.shape = 0;
             } else {
-                r->shape = 1;
+                r.shape = 1;
             }
 
             if (newSpot->prevMethod == "hide") {
-                r->prevMethod = 0;
+                r.prevMethod = 0;
             } else {
-                r->prevMethod = 1;
+                r.prevMethod = 1;
             }
 
 
             if (newSpot->spotMethod == "norm") {
-                r->spotMethod = 0;
+                r.spotMethod = 0;
             } else if(newSpot->spotMethod == "exc") {
-                r->spotMethod = 1;
+                r.spotMethod = 1;
             } else if(newSpot->spotMethod == "full") {
-                r->spotMethod = 2;
+                r.spotMethod = 2;
             }
 
-            r->sensiexclu = newSpot->sensiexclu;
-            r->structexclu = newSpot->structexclu;
+            r.sensiexclu = newSpot->sensiexclu;
+            r.structexclu = newSpot->structexclu;
 
             if (newSpot->shapeMethod == "IND") {
-                r->shapeMethod = 0;
+                r.shapeMethod = 0;
             } else if (newSpot->shapeMethod == "SYM") {
-                r->shapeMethod = 1;
+                r.shapeMethod = 1;
             } else if (newSpot->shapeMethod == "INDSL") {
-                r->shapeMethod = 2;
+                r.shapeMethod = 2;
             } else {
-                r->shapeMethod = 3;
+                r.shapeMethod = 3;
+            }
+
+            if (newSpot->avoidgamutMethod == "NONE") {
+                r.avoidgamutMethod = 0;
+            } else if (newSpot->avoidgamutMethod == "LAB") {
+                r.avoidgamutMethod = 1;
+            } else if (newSpot->avoidgamutMethod == "XYZ") {
+                r.avoidgamutMethod = 2;
+            } else if (newSpot->avoidgamutMethod == "XYZREL") {
+                r.avoidgamutMethod = 3;
+            } else if (newSpot->avoidgamutMethod == "MUNS") {
+                r.avoidgamutMethod = 4;
             }
 
             // Calculate spot size and center position according to preview area
@@ -459,65 +482,63 @@ void Locallab::write(rtengine::procparams::ProcParams* pp, ParamsEdited* pedited
                 }
             }
 
-            r->locX = newSpot->loc.at(0);
-            r->locXL = newSpot->loc.at(1);
-            r->locY = newSpot->loc.at(2);
-            r->locYT = newSpot->loc.at(3);
-            r->centerX = newSpot->centerX;
-            r->centerY = newSpot->centerY;
+            r.locX = newSpot->loc.at(0);
+            r.locXL = newSpot->loc.at(1);
+            r.locY = newSpot->loc.at(2);
+            r.locYT = newSpot->loc.at(3);
+            r.centerX = newSpot->centerX;
+            r.centerY = newSpot->centerY;
 
-            r->circrad = newSpot->circrad;
+            r.circrad = newSpot->circrad;
 
             if (newSpot->qualityMethod == "enh") {
-                r->qualityMethod = 0;
+                r.qualityMethod = 0;
             } else {
-                r->qualityMethod = 1;
+                r.qualityMethod = 1;
             }
 
-            r->transit = newSpot->transit;
-            r->transitweak = newSpot->transitweak;
-            r->transitgrad = newSpot->transitgrad;
-            r->feather = newSpot->feather;
-            r->struc = newSpot->struc;
-            r->thresh = newSpot->thresh;
-            r->iter = newSpot->iter;
-            r->balan = newSpot->balan;
-            r->balanh = newSpot->balanh;
-            r->colorde = newSpot->colorde;
-            r->colorscope = newSpot->colorscope;
-            r->avoidrad = newSpot->avoidrad;
-            r->hishow = newSpot->hishow;
-            r->activ = newSpot->activ;
-            r->avoid = newSpot->avoid;
-            r->avoidmun = newSpot->avoidmun;
-            r->blwh = newSpot->blwh;
-            r->recurs = newSpot->recurs;
-            r->laplac = newSpot->laplac;
-            r->deltae = newSpot->deltae;
-            r->scopemask = newSpot->scopemask;
-            r->denoichmask = newSpot->denoichmask;
-            r->shortc = newSpot->shortc;
-            r->lumask = newSpot->lumask;
-            //r->savrest = newSpot->savrest;
+            r.transit = newSpot->transit;
+            r.transitweak = newSpot->transitweak;
+            r.transitgrad = newSpot->transitgrad;
+            r.feather = newSpot->feather;
+            r.struc = newSpot->struc;
+            r.thresh = newSpot->thresh;
+            r.iter = newSpot->iter;
+            r.balan = newSpot->balan;
+            r.balanh = newSpot->balanh;
+            r.colorde = newSpot->colorde;
+            r.colorscope = newSpot->colorscope;
+            r.avoidrad = newSpot->avoidrad;
+            r.hishow = newSpot->hishow;
+            r.activ = newSpot->activ;
+            r.blwh = newSpot->blwh;
+            r.recurs = newSpot->recurs;
+            r.laplac = newSpot->laplac;
+            r.deltae = newSpot->deltae;
+            r.scopemask = newSpot->scopemask;
+            r.denoichmask = newSpot->denoichmask;
+            r.shortc = newSpot->shortc;
+            r.lumask = newSpot->lumask;
+            //r.savrest = newSpot->savrest;
 
             if (newSpot->complexMethod == "sim") {
-                r->complexMethod = 0;
+                r.complexMethod = 0;
             } else  if (newSpot->complexMethod == "mod") {
-                r->complexMethod = 1;
+                r.complexMethod = 1;
             } else  if (newSpot->complexMethod == "all") {
-                r->complexMethod = 2;
+                r.complexMethod = 2;
             }
 
             if (newSpot->wavMethod == "D2") {
-                r->wavMethod = 0;
+                r.wavMethod = 0;
             } else if (newSpot->wavMethod == "D4") {
-                r->wavMethod = 1;
+                r.wavMethod = 1;
             } else if (newSpot->wavMethod == "D6") {
-                r->wavMethod = 2;
+                r.wavMethod = 2;
             } else if (newSpot->wavMethod == "D10") {
-                r->wavMethod = 3;
+                r.wavMethod = 3;
             } else if (newSpot->wavMethod == "D14") {
-                r->wavMethod = 4;
+                r.wavMethod = 4;
             }
 
             expsettings->addControlSpot(r);
@@ -562,6 +583,7 @@ void Locallab::write(rtengine::procparams::ProcParams* pp, ParamsEdited* pedited
             // Note: No need to manage pedited as batch mode is deactivated for Locallab
 
             break;
+        }
 
         case (ControlSpotPanel::SpotDeletion): // Spot deletion event
             // Get deleted spot index in ProcParams and update it
@@ -681,6 +703,19 @@ void Locallab::write(rtengine::procparams::ProcParams* pp, ParamsEdited* pedited
 
                 expreti.updateMinMax(cdma, cdmin, mini, maxi, Tmean, Tsigma, Tmin, Tmax);
             }
+            // Update Locallab Denoise tool lum/chro
+            if (pp->locallab.selspot < (int) denoiselc.size()) {
+                const double highres = denoiselc.at(pp->locallab.selspot).highres;
+                const double nres = denoiselc.at(pp->locallab.selspot).nres;
+                const double highres46 = denoiselc.at(pp->locallab.selspot).highres46;
+                const double nres46 = denoiselc.at(pp->locallab.selspot).nres46;
+                const  double Lhighres = denoiselc.at(pp->locallab.selspot).Lhighres;
+                const double Lnres = denoiselc.at(pp->locallab.selspot).Lnres;
+                const double Lhighres46 = denoiselc.at(pp->locallab.selspot).Lhighres46;
+                const double Lnres46 = denoiselc.at(pp->locallab.selspot).Lnres46;
+
+                expblur.updatedenlc(highres, nres, highres46, nres46, Lhighres, Lnres, Lhighres46, Lnres46);
+            }
 
             // Update default values according to selected spot
             setDefaults(pp, pedited);
@@ -689,7 +724,7 @@ void Locallab::write(rtengine::procparams::ProcParams* pp, ParamsEdited* pedited
 
             break;
 
-        case (ControlSpotPanel::SpotDuplication): // Spot duplication event
+        case (ControlSpotPanel::SpotDuplication): { // Spot duplication event
             newSpot = nullptr;
             spotIndex = expsettings->getSelectedSpot();
 
@@ -705,43 +740,55 @@ void Locallab::write(rtengine::procparams::ProcParams* pp, ParamsEdited* pedited
             }
 
             // Spot creation (initialization at currently selected spot)
-            r = new ControlSpotPanel::SpotRow();
-            r->name = newSpot->name = newSpot->name + " - " + M("TP_LOCALLAB_DUPLSPOTNAME");
-            r->isvisible = newSpot->isvisible;
+            ControlSpotPanel::SpotRow r;
+            r.name = newSpot->name = newSpot->name + " - " + M("TP_LOCALLAB_DUPLSPOTNAME");
+            r.isvisible = newSpot->isvisible;
 
             if (newSpot->shape == "ELI") {
-                r->shape = 0;
+                r.shape = 0;
             } else {
-                r->shape = 1;
+                r.shape = 1;
             }
 
             if (newSpot->prevMethod == "hide") {
-                r->prevMethod = 0;
+                r.prevMethod = 0;
             } else {
-                r->prevMethod = 1;
+                r.prevMethod = 1;
             }
 
             if (newSpot->spotMethod == "norm") {
-                r->spotMethod = 0;
+                r.spotMethod = 0;
             } else if (newSpot->spotMethod == "exc") {
-                r->spotMethod = 1;
+                r.spotMethod = 1;
             } else if (newSpot->spotMethod == "full") {
-                r->spotMethod = 2;
+                r.spotMethod = 2;
             }
 
-            r->sensiexclu = newSpot->sensiexclu;
-            r->structexclu = newSpot->structexclu;
+            r.sensiexclu = newSpot->sensiexclu;
+            r.structexclu = newSpot->structexclu;
 
             if (newSpot->shapeMethod == "IND") {
-                r->shapeMethod = 0;
+                r.shapeMethod = 0;
             } else if (newSpot->shapeMethod == "SYM") {
-                r->shapeMethod = 1;
+                r.shapeMethod = 1;
             } else if (newSpot->shapeMethod == "INDSL") {
-                r->shapeMethod = 2;
+                r.shapeMethod = 2;
             } else {
-                r->shapeMethod = 3;
+                r.shapeMethod = 3;
             }
             //printf("n0=%f n1=%f n2=%f n3=%f\n", (double) newSpot->loc.at(0), (double) newSpot->loc.at(1), (double) newSpot->loc.at(2), (double) newSpot->loc.at(3));
+            if (newSpot->avoidgamutMethod == "NONE") {
+                r.avoidgamutMethod = 0;
+            } else if (newSpot->avoidgamutMethod == "LAB") {
+                r.avoidgamutMethod = 1;
+            } else if (newSpot->avoidgamutMethod== "XYZ") {
+                r.avoidgamutMethod = 2;
+            } else if (newSpot->avoidgamutMethod== "XYZREL") {
+                r.avoidgamutMethod = 3;
+             } else if (newSpot->avoidgamutMethod== "MUNS") {
+                r.avoidgamutMethod = 4;
+           } 
+            //printf("n0=%f n1=%f n2=%f n3=%f\n", (double) newSpot->loc.at(0), (double) newSpot->loc.at(1), (double) newSpot->loc.at(2), (double) newSpot->loc.at(3));
 
             // Calculate spot size and center position according to preview area
             if (provider && !batchMode) {
@@ -763,72 +810,71 @@ void Locallab::write(rtengine::procparams::ProcParams* pp, ParamsEdited* pedited
                 }
             }
 
-            if(r->spotMethod != 2) {
-                r->locX = newSpot->loc.at(0);
-                r->locXL = newSpot->loc.at(1);
-                r->locY = newSpot->loc.at(2);
-                r->locYT = newSpot->loc.at(3);
+            if(r.spotMethod != 2) {
+                r.locX = newSpot->loc.at(0);
+                r.locXL = newSpot->loc.at(1);
+                r.locY = newSpot->loc.at(2);
+                r.locYT = newSpot->loc.at(3);
             } else {
-                r->locX = 3000.;
-                r->locXL = 3000.;
-                r->locY = 3000.;
-                r->locYT = 3000.;
+                r.locX = 3000.;
+                r.locXL = 3000.;
+                r.locY = 3000.;
+                r.locYT = 3000.;
             }
 
-            r->centerX = newSpot->centerX;
-            r->centerY = newSpot->centerY;
+            r.centerX = newSpot->centerX;
+            r.centerY = newSpot->centerY;
 
-            r->circrad = newSpot->circrad;
+            r.circrad = newSpot->circrad;
 
             if (newSpot->qualityMethod == "enh") {
-                r->qualityMethod = 0;
+                r.qualityMethod = 0;
             } else {
-                r->qualityMethod = 1;
+                r.qualityMethod = 1;
             }
 
-            r->transit = newSpot->transit;
-            r->transitweak = newSpot->transitweak;
-            r->transitgrad = newSpot->transitgrad;
-            r->feather = newSpot->feather;
-            r->struc = newSpot->struc;
-            r->thresh = newSpot->thresh;
-            r->iter = newSpot->iter;
-            r->balan = newSpot->balan;
-            r->balanh = newSpot->balanh;
-            r->colorde = newSpot->colorde;
-            r->colorscope = newSpot->colorscope;
-            r->avoidrad = newSpot->avoidrad;
-            r->activ = newSpot->activ;
-            r->avoid = newSpot->avoid;
-            r->avoidmun = newSpot->avoidmun;
-            r->blwh = newSpot->blwh;
-            r->recurs = newSpot->recurs;
-            r->laplac = newSpot->laplac;
-            r->deltae = newSpot->deltae;
-            r->scopemask = newSpot->scopemask;
-            r->denoichmask = newSpot->denoichmask;
-            r->shortc = newSpot->shortc;
-            r->lumask = newSpot->lumask;
-            //r->savrest = newSpot->savrest;
+            r.transit = newSpot->transit;
+            r.transitweak = newSpot->transitweak;
+            r.transitgrad = newSpot->transitgrad;
+            r.feather = newSpot->feather;
+            r.struc = newSpot->struc;
+            r.thresh = newSpot->thresh;
+            r.iter = newSpot->iter;
+            r.balan = newSpot->balan;
+            r.balanh = newSpot->balanh;
+            r.colorde = newSpot->colorde;
+            r.colorscope = newSpot->colorscope;
+            r.avoidrad = newSpot->avoidrad;
+            r.hishow = newSpot->hishow;
+            r.activ = newSpot->activ;
+            r.blwh = newSpot->blwh;
+            r.recurs = newSpot->recurs;
+            r.laplac = newSpot->laplac;
+            r.deltae = newSpot->deltae;
+            r.scopemask = newSpot->scopemask;
+            r.denoichmask = newSpot->denoichmask;
+            r.shortc = newSpot->shortc;
+            r.lumask = newSpot->lumask;
+            //r.savrest = newSpot->savrest;
 
             if (newSpot->complexMethod == "sim") {
-                r->complexMethod = 0;
+                r.complexMethod = 0;
             } else  if (newSpot->complexMethod == "mod") {
-                r->complexMethod = 1;
+                r.complexMethod = 1;
             } else  if (newSpot->complexMethod == "all") {
-                r->complexMethod = 2;
+                r.complexMethod = 2;
             }
 
             if (newSpot->wavMethod == "D2") {
-                r->wavMethod = 0;
+                r.wavMethod = 0;
             } else if (newSpot->wavMethod == "D4") {
-                r->wavMethod = 1;
+                r.wavMethod = 1;
             } else if (newSpot->wavMethod == "D6") {
-                r->wavMethod = 2;
+                r.wavMethod = 2;
             } else if (newSpot->wavMethod == "D10") {
-                r->wavMethod = 3;
+                r.wavMethod = 3;
             } else if (newSpot->wavMethod == "D14") {
-                r->wavMethod = 4;
+                r.wavMethod = 4;
             }
 
             expsettings->addControlSpot(r);
@@ -870,9 +916,10 @@ void Locallab::write(rtengine::procparams::ProcParams* pp, ParamsEdited* pedited
             // Note: No need to manage pedited as batch mode is deactivated for Locallab
 
             break;
+        }
 
-        case (ControlSpotPanel::SpotAllVisibilityChanged): // Event when updating visibility of all spots
-            r = expsettings->getSpot(expsettings->getSelectedSpot());
+        case (ControlSpotPanel::SpotAllVisibilityChanged): { // Event when updating visibility of all spots
+            const auto r = expsettings->getSpot(expsettings->getSelectedSpot());
 
             // ProcParams update
             for (size_t i = 0; i < pp->locallab.spots.size(); i++) {
@@ -882,10 +929,11 @@ void Locallab::write(rtengine::procparams::ProcParams* pp, ParamsEdited* pedited
             // Note: No need to manage pedited as batch mode is deactivated for Locallab
 
             break;
+        }
 
         default: // Spot or locallab GUI updated
             if (pp->locallab.spots.size() > 0) {
-                r = expsettings->getSpot(expsettings->getSelectedSpot());
+                const auto r = expsettings->getSpot(expsettings->getSelectedSpot());
 
                 // ProcParams update
                 if (pp->locallab.selspot < (int)pp->locallab.spots.size()) {
@@ -927,6 +975,18 @@ void Locallab::write(rtengine::procparams::ProcParams* pp, ParamsEdited* pedited
                         pp->locallab.spots.at(pp->locallab.selspot).shapeMethod = "SYMSL";
                     }
 
+                    if (r->avoidgamutMethod == 0) {
+                        pp->locallab.spots.at(pp->locallab.selspot).avoidgamutMethod = "NONE";
+                    } else if (r->avoidgamutMethod == 1) {
+                        pp->locallab.spots.at(pp->locallab.selspot).avoidgamutMethod = "LAB";
+                    } else if (r->avoidgamutMethod == 2) {
+                        pp->locallab.spots.at(pp->locallab.selspot).avoidgamutMethod = "XYZ";
+                    } else if (r->avoidgamutMethod == 3) {
+                        pp->locallab.spots.at(pp->locallab.selspot).avoidgamutMethod = "XYZREL";
+                    } else if (r->avoidgamutMethod == 4) {
+                        pp->locallab.spots.at(pp->locallab.selspot).avoidgamutMethod = "MUNS";
+                    } 
+
                     pp->locallab.spots.at(pp->locallab.selspot).loc.at(0) = r->locX;
                     pp->locallab.spots.at(pp->locallab.selspot).loc.at(1) = r->locXL;
                     pp->locallab.spots.at(pp->locallab.selspot).loc.at(2) = r->locY;
@@ -955,8 +1015,6 @@ void Locallab::write(rtengine::procparams::ProcParams* pp, ParamsEdited* pedited
                     pp->locallab.spots.at(pp->locallab.selspot).avoidrad = r->avoidrad;
                     pp->locallab.spots.at(pp->locallab.selspot).hishow = r->hishow;
                     pp->locallab.spots.at(pp->locallab.selspot).activ = r->activ;
-                    pp->locallab.spots.at(pp->locallab.selspot).avoid = r->avoid;
-                    pp->locallab.spots.at(pp->locallab.selspot).avoidmun = r->avoidmun;
                     pp->locallab.spots.at(pp->locallab.selspot).blwh = r->blwh;
                     pp->locallab.spots.at(pp->locallab.selspot).recurs = r->recurs;
                     pp->locallab.spots.at(pp->locallab.selspot).laplac = r->laplac;
@@ -1052,6 +1110,28 @@ void Locallab::minmaxChanged(const std::vector<locallabRetiMinMax> &minmax, int
     }
 }
 
+void Locallab::denChanged(const std::vector<locallabDenoiseLC> &denlc, int selspot)
+{
+    // Saving transmitted min/max data
+    denoiselc = denlc;
+    
+    //Update Locallab Denoise tool lum chro
+    if (selspot < (int) denoiselc.size()) {
+        const double highres = denoiselc.at(selspot).highres;
+        const double nres = denoiselc.at(selspot).nres;
+        const double highres46 = denoiselc.at(selspot).highres46;
+        const double nres46 = denoiselc.at(selspot).nres46;
+        const  double Lhighres = denoiselc.at(selspot).Lhighres;
+        const double Lnres = denoiselc.at(selspot).Lnres;
+        const double Lhighres46 = denoiselc.at(selspot).Lhighres46;
+        const double Lnres46 = denoiselc.at(selspot).Lnres46;
+
+        expblur.updatedenlc(highres, nres, highres46, nres46, Lhighres, Lnres, Lhighres46, Lnres46);
+    }
+    
+}
+
+
 void Locallab::logencodChanged(const float blackev, const float whiteev, const float sourceg, const float sourceab, const float targetg, const bool autocomput, const bool autocie, const float jz1)
 {
     // Update Locallab Log Encoding and Ciecam accordingly
