@@ -345,6 +345,7 @@ private:
     MyExpander* const expfat;
     Adjuster* const fatamount;
     Adjuster* const fatdetail;
+    Gtk::CheckButton* const fatsatur;
     Gtk::CheckButton* const norm;
     Adjuster* const fatlevel;
     Adjuster* const fatanchor;
@@ -395,8 +396,9 @@ private:
     Adjuster* const angmaskexp;
     CurveEditorGroup* const mask2expCurveEditorG;
     DiagonalCurveEditor* const Lmaskexpshape;
+    rtengine::ProcEvent Evlocallabtmosatur;
 
-    sigc::connection expMethodConn, exnoiseMethodConn, inversexConn, normConn, showmaskexpMethodConn, showmaskexpMethodConninv, enaExpMaskConn, enaExpMaskaftConn;
+    sigc::connection expMethodConn, exnoiseMethodConn, inversexConn, normConn, fatsaturConn, showmaskexpMethodConn, showmaskexpMethodConninv, enaExpMaskConn, enaExpMaskaftConn;
 
 public:
     LocallabExposure();
@@ -429,6 +431,7 @@ private:
     void exnoiseMethodChanged();
     void inversexChanged();
     void normChanged();
+    void fatsaturChanged();
     void showmaskexpMethodChanged();
     void showmaskexpMethodChangedinv();
     void enaExpMaskChanged();
@@ -451,6 +454,7 @@ private:
     Adjuster* const reparsh;
     const std::array<Adjuster*, 5> multipliersh;
     Adjuster* const detailSH;
+    Adjuster* const tePivot;
     Adjuster* const highlights;
     Adjuster* const h_tonalwidth;
     Adjuster* const shadows;
@@ -492,6 +496,8 @@ private:
     Adjuster* const fatamountSH;
     Adjuster* const fatanchorSH;
 
+    rtengine::ProcEvent EvlocallabTePivot;
+
     sigc::connection shMethodConn, inversshConn, showmaskSHMethodConn, showmaskSHMethodConninv, enaSHMaskConn;
 
 public:
@@ -707,8 +713,17 @@ private:
     Gtk::CheckButton* const activlum;
     MyExpander* const expdenoise;
     MyComboBoxText* const quamethod;
+    MyExpander* const expdenoisenl;
+    MyExpander* const expdenoiselum;
+    MyExpander* const expdenoisech;
     CurveEditorGroup* const LocalcurveEditorwavden;
     FlatCurveEditor* const wavshapeden;
+    Gtk::Label* const lCLabels;
+    Gtk::Label* const lumLabels;
+    Gtk::Label* const lum46Labels;
+    Gtk::Label* const chroLabels;
+    Gtk::Label* const chro46Labels;
+    
     MyExpander* const expdenoise1;
     Gtk::Label* const maskusable;
     Gtk::Label* const maskunusable;
@@ -746,7 +761,7 @@ private:
     
     Gtk::CheckButton* const invmaskd;
     Gtk::CheckButton* const invmask;
-    Gtk::Frame* const nlFrame;
+    Gtk::Frame* const prevFrame;
     Adjuster* const nlstr;
     Adjuster* const nldet;
     Adjuster* const nlpat;
@@ -788,6 +803,8 @@ private:
 public:
     LocallabBlur();
     ~LocallabBlur();
+    void updatedenlc(const double highres, const double nres, const double highres46, const double nres46, const double Lhighres, const double Lnres, const double Lhighres46, const double Lnres46);
+
 
     bool isMaskViewActive() override;
     void resetMaskView() override;
