@@ -18,6 +18,7 @@
  */
 #pragma once
 
+#include <array>
 #include <vector>
 
 namespace rtengine
@@ -52,6 +53,7 @@ struct ToneCurveParamsEdited {
     bool shcompr;
     bool hlcompr;
     bool hlbl;
+    bool hlth;
     bool hlcomprthresh;
     bool autoexp;
     bool clip;
@@ -109,7 +111,7 @@ struct LCurveParamsEdited {
     bool brightness;
     bool contrast;
     bool chromaticity;
-    bool avoidcolorshift;
+    bool gamutmunselmethod;
     bool rstprotection;
     bool lcurve;
     bool acurve;
@@ -245,7 +247,16 @@ struct WBParamsEdited {
     bool temperature;
     bool green;
     bool equal;
+    bool observer;
     bool tempBias;
+    bool itcwb_rgreen;
+    bool itcwb_nopurple;
+    bool itcwb_alg;
+    bool itcwb_prim;
+    bool itcwb_sampling;
+    bool itcwb_green;
+    bool compat_version;
+
 };
 
 struct DefringeParamsEdited {
@@ -305,7 +316,6 @@ struct ColorAppearanceParamsEdited {
     bool ybout;
     bool tempsc;
     bool greensc;
-    bool presetcat02;
 };
 
 struct DirPyrDenoiseParamsEdited {
@@ -317,6 +327,7 @@ struct DirPyrDenoiseParamsEdited {
     bool chroma;
     bool redchro;
     bool bluechro;
+    bool gain;
     bool gamma;
     bool lcurve;
     bool cccurve;
@@ -358,6 +369,14 @@ struct SHParamsEdited {
     bool lab;
 };
 
+struct ToneEqualizerParamsEdited {
+    bool enabled;
+    std::array<bool, 5> bands;
+    bool regularization;
+    bool show_colormap;
+    bool pivot;
+};
+
 struct CropParamsEdited {
     bool enabled;
     bool x;
@@ -403,6 +422,7 @@ public:
         bool structexclu;
         bool struc;
         bool shapeMethod;
+        bool avoidgamutMethod;
         bool loc;
         bool centerX;
         bool centerY;
@@ -422,8 +442,6 @@ public:
         bool transitgrad;
         bool hishow;
         bool activ;
-        bool avoid;
-        bool avoidmun;
         bool blwh;
         bool recurs;
         bool laplac;
@@ -549,6 +567,7 @@ public:
         bool fatdetail;
         bool fatanchor;
         bool fatlevel;
+        bool fatsatur;
         bool recothrese;
         bool lowthrese;
         bool higthrese;
@@ -580,6 +599,7 @@ public:
         bool slomaskSH;
         bool lapmaskSH;
         bool detailSH;
+        bool tePivot;
         bool reparsh;
         bool LmaskSHcurve;
         bool fatamountSH;
@@ -1231,6 +1251,7 @@ struct ColorManagementParamsEdited {
     bool bluy;
     bool preser;
     bool fbw;
+    bool gamut;
     bool labgridcieALow;
     bool labgridcieBLow;
     bool labgridcieAHigh;
@@ -1489,6 +1510,7 @@ struct RAWParamsEdited {
     bool df_autoselect;
     bool ff_file;
     bool ff_AutoSelect;
+    bool ff_FromMetaData;
     bool ff_BlurRadius;
     bool ff_BlurType;
     bool ff_AutoClipControl;
@@ -1509,6 +1531,7 @@ struct RAWParamsEdited {
 
 struct MetaDataParamsEdited {
     bool mode;
+    bool exifKeys;
 };
 
 struct FilmNegativeParamsEdited {
@@ -1545,6 +1568,7 @@ struct ParamsEdited {
     FattalToneMappingParamsEdited fattal;
     ImpulseDenoiseParamsEdited impulseDenoise;
     SHParamsEdited sh;
+    ToneEqualizerParamsEdited toneEqualizer;
     CropParamsEdited crop;
     CoarseTransformParamsEdited coarse;
     CommonTransformParamsEdited commonTrans;
