@@ -18,7 +18,7 @@
  */
 #include "placesbrowser.h"
 
-#ifdef WIN32
+#ifdef _WIN32
 #include <windows.h>
 #include <shlobj.h>
 #include <Shlwapi.h>
@@ -106,9 +106,32 @@ void PlacesBrowser::refreshPlacesList ()
 {
     placesModel->clear ();
 
+    // append favorites
+    for (size_t i = 0; i < options.favoriteDirs.size(); i++) {
+        Glib::RefPtr<Gio::File> fav = Gio::File::create_for_path (options.favoriteDirs[i]);
+
+        if (fav && fav->query_exists()) {
+            try {
+                if (auto info = fav->query_info ()) {
+                    Gtk::TreeModel::Row newrow = *(placesModel->append());
+                    newrow[placesColumns.label] = info->get_display_name ();
+                    newrow[placesColumns.icon]  = info->get_icon ();
+                    newrow[placesColumns.root]  = fav->get_parse_name ();
+                    newrow[placesColumns.type]  = 5;
+                    newrow[placesColumns.rowSeparator] = false;
+                }
+            } catch(Gio::Error&) {}
+        }
+    }
+
     // append home directory
     Glib::RefPtr<Gio::File> hfile = Gio::File::create_for_path (userHomeDir());  // Will send back "My documents" on Windows now, which has no restricted access
 
+    if (!placesModel->children().empty()) {
+        Gtk::TreeModel::Row newrow = *(placesModel->append());
+        newrow[placesColumns.rowSeparator] = true;
+    }
+
     if (hfile && hfile->query_exists()) {
         try {
             if (auto info = hfile->query_info ()) {
@@ -208,7 +231,7 @@ void PlacesBrowser::refreshPlacesList ()
     // (Drives in Windows)
     std::vector<Glib::RefPtr<Gio::Mount> > mounts = vm->get_mounts ();
 
-#ifdef WIN32
+#ifdef _WIN32
     // on Windows, it's usual to sort by drive letter, not by name
     std::sort (mounts.begin(), mounts.end(), compareMountByRoot);
 #endif
@@ -223,29 +246,6 @@ void PlacesBrowser::refreshPlacesList ()
             newrow[placesColumns.rowSeparator] = false;
         }
     }
-
-    // append favorites
-    if (!placesModel->children().empty()) {
-        Gtk::TreeModel::Row newrow = *(placesModel->append());
-        newrow[placesColumns.rowSeparator] = true;
-    }
-
-    for (size_t i = 0; i < options.favoriteDirs.size(); i++) {
-        Glib::RefPtr<Gio::File> fav = Gio::File::create_for_path (options.favoriteDirs[i]);
-
-        if (fav && fav->query_exists()) {
-            try {
-                if (auto info = fav->query_info ()) {
-                    Gtk::TreeModel::Row newrow = *(placesModel->append());
-                    newrow[placesColumns.label] = info->get_display_name ();
-                    newrow[placesColumns.icon]  = info->get_icon ();
-                    newrow[placesColumns.root]  = fav->get_parse_name ();
-                    newrow[placesColumns.type]  = 5;
-                    newrow[placesColumns.rowSeparator] = false;
-                }
-            } catch(Gio::Error&) {}
-        }
-    }
 }
 
 bool PlacesBrowser::rowSeparatorFunc (const Glib::RefPtr<Gtk::TreeModel>& model, const Gtk::TreeModel::iterator& iter)
@@ -353,7 +353,7 @@ void PlacesBrowser::delPressed ()
 
 Glib::ustring PlacesBrowser::userHomeDir ()
 {
-#ifdef WIN32
+#ifdef _WIN32
 
     // get_home_dir crashes on some Windows configurations,
     // so we rather use the safe native functions here.
@@ -378,7 +378,7 @@ Glib::ustring PlacesBrowser::userHomeDir ()
 
 Glib::ustring PlacesBrowser::userPicturesDir ()
 {
-#ifdef WIN32
+#ifdef _WIN32
 
     // get_user_special_dir crashes on some Windows configurations,
     // so we rather use the safe native functions here.
