@@ -20,12 +20,20 @@
  */
 #pragma once
 
+#include "threadutils.h"
+
+#include <memory>
 #include <vector>
 
+#include <glibmm/refptr.h>
 #include <glibmm/ustring.h>
-
 #include <sigc++/signal.h>
 
+namespace Gio
+{
+class Icon;
+}
+
 namespace Gtk
 {
 
@@ -33,6 +41,8 @@ class Grid;
 class Menu;
 class Button;
 class ImageMenuItem;
+class RadioButtonGroup;
+class Widget;
 
 }
 
@@ -52,10 +62,15 @@ public:
 
     explicit PopUpCommon (Gtk::Button* button, const Glib::ustring& label = "");
     virtual ~PopUpCommon ();
-    bool addEntry (const Glib::ustring& fileName, const Glib::ustring& label);
+    bool addEntry (const Glib::ustring& fileName, const Glib::ustring& label, Gtk::RadioButtonGroup* radioGroup = nullptr);
+    bool insertEntry(int position, const Glib::ustring& fileName, const Glib::ustring& label, Gtk::RadioButtonGroup* radioGroup = nullptr);
+    bool insertEntry(int position, const Glib::RefPtr<const Gio::Icon>& gIcon, const Glib::ustring& label, Gtk::RadioButtonGroup* radioGroup = nullptr);
+    /// Sets the button image to show when there are no entries.
+    void setEmptyImage(const Glib::ustring &fileName);
     int getEntryCount () const;
     bool setSelected (int entryNum);
     int  getSelected () const;
+    void removeEntry(int position);
     void setButtonHint();
     void show ();
     void set_tooltip_text (const Glib::ustring &text);
@@ -65,16 +80,24 @@ private:
     type_signal_changed messageChanged;
     type_signal_item_selected messageItemSelected;
 
+    Glib::ustring emptyImageFilename;
+    std::vector<Glib::RefPtr<const Gio::Icon>> imageIcons;
     std::vector<Glib::ustring> imageFilenames;
     std::vector<const RTImage*> images;
     Glib::ustring buttonHint;
     RTImage* buttonImage;
     Gtk::Grid* imageContainer;
-    Gtk::Menu* menu;
+    std::unique_ptr<Gtk::Menu> menu;
     Gtk::Button* button;
+    Gtk::Button* arrowButton;
     int selected;
     bool hasMenu;
+    MyMutex entrySelectionMutex;
 
+    void changeImage(int position);
+    void changeImage(const Glib::ustring& fileName, const Glib::RefPtr<const Gio::Icon>& gIcon);
+    void entrySelected(Gtk::Widget* menuItem);
+    bool insertEntryImpl(int position, const Glib::ustring& fileName, const Glib::RefPtr<const Gio::Icon>& gIcon, RTImage* image, const Glib::ustring& label, Gtk::RadioButtonGroup* radioGroup);
     void showMenu(GdkEventButton* event);
 
 protected:
