@@ -21,6 +21,7 @@
 #include <gtkmm.h>
 
 #include "adjuster.h"
+#include "checkbox.h"
 #include "guiutils.h"
 #include "toolpanel.h"
 #include "wbprovider.h"
@@ -35,7 +36,7 @@ public:
     virtual void spotWBRequested(int size) = 0;
 };
 
-class WhiteBalance final : public ToolParamBlock, public AdjusterListener, public FoldableToolPanel, public rtengine::AutoWBListener
+class WhiteBalance final : public ToolParamBlock, public AdjusterListener, public CheckBoxListener, public FoldableToolPanel, public rtengine::AutoWBListener
 {
 
     enum WB_LabelType {
@@ -45,6 +46,9 @@ class WhiteBalance final : public ToolParamBlock, public AdjusterListener, publi
 
 private:
     Gtk::Label*  StudLabel;
+    Gtk::Label*  PatchLabel;
+    Gtk::Label*  PatchlevelLabel;
+    Gtk::Label*  mulLabel;
 
 protected:
     class MethodColumns : public Gtk::TreeModel::ColumnRecord
@@ -60,6 +64,11 @@ protected:
             add(colId);
         }
     };
+    
+    rtengine::ProcEvent EvWBObserver10;
+    rtengine::ProcEvent EvWBitcwbprim;
+    rtengine::ProcEvent EvWBitcwbalg;
+    rtengine::ProcEvent EvWBitcwgreen;
 
     static Glib::RefPtr<Gdk::Pixbuf> wbPixbufs[rtengine::toUnderlying(rtengine::procparams::WBEntry::Type::CUSTOM) + 1];
     Glib::RefPtr<Gtk::TreeStore> refTreeModel;
@@ -71,6 +80,14 @@ protected:
     Adjuster* green;
     Adjuster* equal;
     Adjuster* tempBias;
+    CheckBox* observer10;
+    Gtk::Frame* itcwbFrame;
+    Gtk::CheckButton* itcwb_alg;
+    MyComboBoxText* itcwb_prim;
+    Adjuster* itcwb_green;
+    std::unique_ptr<Adjuster> compatVersionAdjuster;
+    
+    bool lastitcwb_alg;
 
     Gtk::Button* spotbutton;
     int opt;
@@ -78,7 +95,7 @@ protected:
     double nextGreen;
     WBProvider *wbp;  // pointer to a ToolPanelCoordinator object, or its subclass BatchToolPanelCoordinator
     SpotWBListener* wblistener;
-    sigc::connection methconn;
+    sigc::connection methconn, itcwb_algconn, itcwb_primconn;
     int custom_temp;
     double custom_green;
     double custom_equal;
@@ -98,6 +115,7 @@ protected:
     std::pair<bool, const rtengine::procparams::WBEntry&> findWBEntry    (const Glib::ustring& label, enum WB_LabelType lblType = WBLT_GUI);
 
 public:
+    static const Glib::ustring TOOL_NAME;
 
     WhiteBalance ();
     ~WhiteBalance () override;
@@ -113,6 +131,7 @@ public:
     void spotPressed ();
     void spotSizeChanged ();
     void adjusterChanged(Adjuster* a, double newval) override;
+    void checkBoxToggled(CheckBox* c, CheckValue newval) override;
     int  getSize ();
     void setWBProvider (WBProvider* p)
     {
@@ -124,8 +143,9 @@ public:
     }
     void setWB (int temp, double green);
     void resetWB ();
-    void WBChanged           (double temp, double green, float studgood) override;
-
+    void WBChanged           (int met, double temp, double green, double rw, double gw, double bw, float temp0, float delta, int bia, int dread, float studgood, float minchrom, int kmin, float histmin, float histmax, AWBMode aWBMode) override;
+    void itcwb_alg_toggled ();
+    void itcwb_prim_changed ();
     void setAdjusterBehavior (bool tempadd, bool greenadd, bool equaladd, bool tempbiasadd);
     void trimValues          (rtengine::procparams::ProcParams* pp) override;
     void enabledChanged() override;
