@@ -9,11 +9,11 @@
 # - GTK_PREFIX
 
 # Formatting
-fNormal="$(tput sgr0)"
-fBold="$(tput bold)"
+fNormal="$(printf "\e[0m")"
+fBold="$(printf "\e[1m")"
 # Colors depend upon the user's terminal emulator color scheme - what is readable for you may be not readable for someone else.
-fMagenta="$(tput setaf 5)"
-fRed="$(tput setaf 1)"
+fMagenta="$(printf "\e[1;35m")"
+fRed="$(printf "\e[1;31m")"
 
 function msg {
     printf "\\n${fBold}-- %s${fNormal}\\n" "${@}"
@@ -120,8 +120,8 @@ minimum_macos_version=${MINIMUM_SYSTEM_VERSION}
 #Out: /opt
 LOCAL_PREFIX="$(cmake .. -L -N | grep LOCAL_PREFIX)"; LOCAL_PREFIX="${LOCAL_PREFIX#*=}"
 
-#In: OSX_UNIVERSAL_URL=https:// etc.
-#Out: https:// etc.
+#In: OSX_UNIVERSAL_URL=file:/// etc.
+#Out: file:/// etc.
 UNIVERSAL_URL="$(cmake .. -L -N | grep OSX_UNIVERSAL_URL)"; UNIVERSAL_URL="${UNIVERSAL_URL#*=}"
 if [[ -n $UNIVERSAL_URL ]]; then
     echo "Universal app is ON. The URL is ${UNIVERSAL_URL}"
@@ -135,8 +135,8 @@ EXPATLIB="$(cmake .. -LA -N | grep pkgcfg_lib_EXPAT_expat)"; pkgcfg_lib_EXPAT_ex
 #Out: Developer ID Application: Doctor Who (1234567890)
 CODESIGNID="$(cmake .. -L -N | grep CODESIGNID)"; CODESIGNID="${CODESIGNID#*=}"
 
-#In: NOTARY:STRING=--username drwho@bbc.com --password abcd-efgh-hijk-lmno
-#Out: --username drwho@bbc.com --password abcd-efgh-hijk-lmno
+#In: NOTARY:STRING="--apple-id drwho@bbc.com --password abcd-efgh-hijk-lmno --team-id ABCDE12345"
+#Out: --apple-id drwho@bbc.com --password abcd-efgh-hijk-lmno --team-id ABCDE12345
 NOTARY="$(cmake .. -L -N | grep NOTARY)"; NOTARY="${NOTARY#*=}"
 
 # In: FANCY_DMG:BOOL=ON
@@ -153,6 +153,13 @@ if [[ -n $NIGHTLY ]]; then
     echo "Nightly/generically-named zip is ON."
 fi
 
+# In: OSX_CONTINUOUS:BOOL=ON
+# Out: ON
+OSX_CONTINUOUS="$(cmake .. -L -N | grep OSX_CONTINUOUS)"; NIGHTLY="${OSX_CONTINUOUS#*=}" && CONTINUOUS="${OSX_CONTINUOUS#*=}"
+if [[ -n $CONTINUOUS ]]; then
+    echo "Continuous/generically-named zip is ON."
+fi
+
 APP="${PROJECT_NAME}.app"
 CONTENTS="${APP}/Contents"
 RESOURCES="${CONTENTS}/Resources"
@@ -191,15 +198,17 @@ lensfunversion=$(pkg-config --modversion lensfun | cut -f3 -d'.')
 if [ $lensfunversion = 95 ]
 then
     ditto ${LOCAL_PREFIX}/share/lensfun/version_2/* "${RESOURCES}/share/lensfun"
+    # Copy liblensfun to Frameworks
+    ditto ${LOCAL_PREFIX}/lib/liblensfun.2.dylib "${CONTENTS}/Frameworks/liblensfun.2.dylib"
+
 else
     ditto ${LOCAL_PREFIX}/share/lensfun/version_1/* "${RESOURCES}/share/lensfun"
+    # Copy liblensfun to Frameworks
+    ditto ${LOCAL_PREFIX}/lib/liblensfun.1.dylib "${CONTENTS}/Frameworks/liblensfun.1.dylib"
 fi
 
-# Copy liblensfun to Frameworks
-ditto ${LOCAL_PREFIX}/lib/liblensfun.2.dylib "${CONTENTS}/Frameworks/liblensfun.2.dylib"
-
 # Copy libomp to Frameworks
-ditto ${LOCAL_PREFIX}/lib/libomp.dylib "${CONTENTS}/Frameworks"
+cp ${LOCAL_PREFIX}/lib/libomp.dylib "${CONTENTS}/Frameworks"
 
 msg "Copying dependencies from ${GTK_PREFIX}."
 CheckLink "${EXECUTABLE}" 2>&1
@@ -207,24 +216,17 @@ CheckLink "${EXECUTABLE}" 2>&1
 # dylib install names
 ModifyInstallNames 2>&1
 
-# Copy libjpeg-turbo ("62") into the app bundle
-ditto ${LOCAL_PREFIX}/lib/libjpeg.62.dylib "${CONTENTS}/Frameworks/libjpeg.62.dylib"
+# Copy libpng16 to the app bundle
+cp ${LOCAL_PREFIX}/lib/libpng16.16.dylib "${CONTENTS}/Frameworks/libpng16.16.dylib"
 
-# Copy libexpat into the app bundle (which is keg-only)
-if [[ -d /usr/local/Cellar/expat ]]; then ditto /usr/local/Cellar/expat/*/lib/libexpat.1.dylib "${CONTENTS}/Frameworks"; else ditto "${EXPATLIB}" "${CONTENTS}/Frameworks/libexpat.1.dylib"; fi
+# Copy graphite to Frameworks
+cp ${LOCAL_PREFIX}/lib/libgraphite2.3.dylib "${CONTENTS}/Frameworks"
 
-# Copy libz into the app bundle
-ditto ${LOCAL_PREFIX}/lib/libz.1.dylib "${CONTENTS}/Frameworks"
-
-# Copy libpng12 & 16 to the app bundle
-ditto ${LOCAL_PREFIX}/lib/libpng16.16.dylib "${CONTENTS}/Frameworks/libpng16.16.dylib"
-ditto ${LOCAL_PREFIX}/lib/libpng12.0.dylib "${CONTENTS}/Frameworks/libpng12.0.dylib"
-
-# Copy libtiff 5 into the app bundle
-ditto ${LOCAL_PREFIX}/lib/libtiff.5.dylib "${CONTENTS}/Frameworks/libtiff.5.dylib"
+# Copy libtiff 6 into the app bundle
+cp ${LOCAL_PREFIX}/lib/libtiff.6.dylib "${CONTENTS}/Frameworks/libtiff.6.dylib"
 
 # Copy libomp to Frameworks
-ditto ${LOCAL_PREFIX}/lib/libomp.dylib "${CONTENTS}/Frameworks"
+cp ${LOCAL_PREFIX}/lib/libomp.dylib "${CONTENTS}/Frameworks"
 
 # Prepare GTK+3 installation
 msg "Copying configuration files from ${GTK_PREFIX}:"
@@ -238,7 +240,7 @@ find -E "${LIB}" -type f -regex '.*\.(a|la|cache)$' | while read -r; do rm "${RE
 # Make Frameworks folder flat
 msg "Flattening the Frameworks folder"
 cp -RL "${LIB}"/gdk-pixbuf-2.0/2*/loaders/* "${LIB}"
-cp "${LIB}"/gtk-3.0/3*/immodules/*.{dylib,so} "${LIB}"
+cp "${LIB}"/gtk-3.0/3*/immodules/*.{dylib,so} "${LIB}" >/dev/null 2>&1
 rm -r "${LIB}"/gtk-3.0
 rm -r "${LIB}"/gdk-pixbuf-2.0
 
@@ -287,7 +289,6 @@ ditto "${PROJECT_SOURCE_DIR}/rtdata/fonts" "${ETC}/fonts"
 
 # App bundle resources
 ditto "${PROJECT_SOURCE_DATA_DIR}/"{rawtherapee,profile}.icns "${RESOURCES}"
-#ditto "${PROJECT_SOURCE_DATA_DIR}/PkgInfo" "${CONTENTS}"
 
 update-mime-database -V  "${RESOURCES}/share/mime"
 cp -RL "${LOCAL_PREFIX}/share/locale" "${RESOURCES}/share/locale"
@@ -311,7 +312,6 @@ for frameworklibs in "${LIB}"/*{dylib,so,cli}; do
 done
 install_name_tool -delete_rpath RawTherapee.app/Contents/Frameworks "${EXECUTABLE}"-cli 2>/dev/null
 install_name_tool -add_rpath /Applications/"${LIB}" "${EXECUTABLE}"-cli 2>/dev/null
-ditto "${EXECUTABLE}"-cli "${APP}"/..
 
 # Merge the app with the other architecture to create the Universal app.
 if [[ -n $UNIVERSAL_URL ]]; then
@@ -319,7 +319,7 @@ if [[ -n $UNIVERSAL_URL ]]; then
     curl -L ${UNIVERSAL_URL} -o univ.zip
     msg "Extracting app."
     unzip univ.zip -d univapp
-    hdiutil attach -mountpoint ./RawTherapeeuniv univapp/*/*dmg
+    hdiutil attach -mountpoint ./RawTherapeeuniv univapp/*dmg
     if [[ $arch = "arm64" ]]; then
         cp -R RawTherapee.app RawTherapee-arm64.app
         minimum_arm64_version=$(f=$(cat RawTherapee-arm64.app/Contents/Resources/AboutThisBuild.txt | grep mmacosx-version); echo "${f#*min=}" | cut -d ' ' -f1)
@@ -339,13 +339,15 @@ if [[ -n $UNIVERSAL_URL ]]; then
     hdiutil unmount ./RawTherapeeuniv
     rm -r univapp
     # Create the fat main RawTherapee binary and move it into the new bundle
-    lipo -create -output RawTherapee RawTherapee-arm64.app/Contents/MacOS/RawTherapee RawTherapee-x86_64.app/Contents/MacOS/RawTherapee
-    mv RawTherapee RawTherapee.app/Contents/MacOS
+    lipo -create -output RawTherapee RawTherapee-arm64.app/Contents/MacOS/rawtherapee RawTherapee-x86_64.app/Contents/MacOS/rawtherapee
+    lipo -create -output rawtherapee-cli RawTherapee-arm64.app/Contents/MacOS/rawtherapee-cli RawTherapee-x86_64.app/Contents/MacOS/rawtherapee-cli
+    mv rawtherapee RawTherapee.app/Contents/MacOS
     # Create all the fat dependencies and move them into the bundle
     for lib in RawTherapee-arm64.app/Contents/Frameworks/* ; do
         lipo -create -output $(basename $lib) RawTherapee-arm64.app/Contents/Frameworks/$(basename $lib) RawTherapee-x86_64.app/Contents/Frameworks/$(basename $lib)
     done
-    sudo mv *cli *so *dylib RawTherapee.app/Contents/Frameworks
+    sudo mv *so *dylib RawTherapee.app/Contents/Frameworks
+    sudo mv *-cli RawTherapee.app/Contents/MacOS
     rm -r RawTherapee-arm64.app
     rm -r RawTherapee-x86_64.app
 else
@@ -358,7 +360,8 @@ fi
 if [[ -n $CODESIGNID ]]; then
     msg "Codesigning Application."
     iconv -f UTF-8 -t ASCII "${PROJECT_SOURCE_DATA_DIR}"/rt.entitlements > "${CMAKE_BUILD_TYPE}"/rt.entitlements
-    mv "${EXECUTABLE}"-cli "${LIB}"
+#    mv "${EXECUTABLE}"-cli "${LIB}"
+    codesign --force --deep --timestamp --strict -v -s "${CODESIGNID}" -i com.rawtherapee.RawTherapee-cli "${APP}"/Contents/MacOS/rawtherapee-cli
     codesign --force --deep --timestamp --strict -v -s "${CODESIGNID}" -i com.rawtherapee.RawTherapee -o runtime --entitlements "${CMAKE_BUILD_TYPE}"/rt.entitlements "${APP}"
     spctl -a -vvvv "${APP}"
 fi
@@ -368,27 +371,7 @@ if [[ -n $NOTARY ]]; then
     msg "Notarizing the application:"
     ditto -c -k --sequesterRsrc --keepParent "${APP}" "${APP}.zip"
     echo "Uploading..."
-    uuid=`xcrun altool --notarize-app --primary-bundle-id "com.rawtherapee.RawTherapee" ${NOTARY} --file "${APP}.zip" 2>&1 | grep 'RequestUUID' | awk '{ print $3 }'`
-    echo "Result= $uuid" # Display identifier string
-    sleep 15
-    while :
-    do
-        fullstatus=`xcrun altool --notarization-info "$uuid" ${NOTARY}  2>&1`  # get the status
-        status1=`echo "$fullstatus" | grep 'Status\:' | awk '{ print $2 }'`
-        if [[ $status1 = "success" ]]; then
-            xcrun stapler staple *app   #  staple the ticket
-            xcrun stapler validate -v *app
-            echo "Notarization success"
-            break
-            elif [[ $status1 = "in" ]]; then
-            echo "Notarization still in progress, sleeping for 15 seconds and trying again"
-            sleep 15
-        else
-            echo "Notarization failed fullstatus below"
-            echo "$fullstatus"
-            exit 1
-        fi
-    done
+    sudo xcrun notarytool submit "${APP}.zip" ${NOTARY} --wait
 fi
 
 function CreateDmg {
@@ -420,21 +403,32 @@ function CreateDmg {
     fi
     
     msg "Creating disk image:"
-    if [[ ! -z $FANCY_DMG ]]; then
+    if [[ $FANCY_DMG == "ON" ]]; then
         echo "Building Fancy .dmg"
-        mkdir "${srcDir}/.background"
-        cp -R "${PROJECT_SOURCE_DATA_DIR}/rtdmg.icns" "${srcDir}/.VolumeIcon.icns"
-        cp -R "${PROJECT_SOURCE_DATA_DIR}/rtdmg-bkgd.png" "${srcDir}/.background/background.png"
-        SetFile -c incC "${srcDir}/.VolumeIcon.icns"
-        create-dmg "${dmg_name}.dmg" "${srcDir}" \
-        --volname "${PROJECT_NAME}_${PROJECT_FULL_VERSION}" \
-        --appname "${PROJECT_NAME}" \
-        --volicon "${srcDir}/.VolumeIcon.icns" \
-        --sandbox-safe \
+        create-dmg \
+        --background ${PROJECT_SOURCE_DATA_DIR}/rtdmg-bkgd.png \
+        --volname ${PROJECT_NAME}_${PROJECT_FULL_VERSION} \
+        --volicon ${PROJECT_SOURCE_DATA_DIR}/rtdmg.icns \
+        --window-pos 72 72 \
+        --window-size 1000 689 \
+        --text-size 16 \
+        --icon-size 80 \
+        --icon LICENSE 810 0 \
+        --icon RawTherapee.app 250 178 \
+        --icon Applications 700 178 \
+        --icon Website.webloc 300 423 \
+        --icon Forum.webloc 420 423 \
+        --icon Report\ Bug.webloc 540 423 \
+        --icon Documentation.webloc 680 423 \
         --no-internet-enable \
-        --eula LICENSE.txt \
+        --eula ${PROJECT_SOURCE_DATA_DIR}/../../LICENSE \
         --hdiutil-verbose \
-        --rez /Library/Developer/CommandLineTools/usr/bin/Rez
+        --hide-extension Website.webloc \
+        --hide-extension Report\ Bug.webloc \
+        --hide-extension Forum.webloc \
+        --hide-extension Documentation.webloc \
+        --filesystem APFS \
+        ${dmg_name}.dmg ${srcDir}
     else
         hdiutil create -format UDBZ -fs HFS+ -srcdir "${srcDir}" -volname "${PROJECT_NAME}_${PROJECT_FULL_VERSION}" "${dmg_name}.dmg"
     fi
@@ -450,39 +444,32 @@ function CreateDmg {
         msg "Notarizing the dmg:"
         zip "${dmg_name}.dmg.zip" "${dmg_name}.dmg"
         echo "Uploading..."
-        uuid=$(xcrun altool --notarize-app --primary-bundle-id "com.rawtherapee" ${NOTARY} --file "${dmg_name}.dmg.zip" 2>&1 | grep 'RequestUUID' | awk '{ print $3 }')
-        echo "dmg Result= ${uuid}" # Display identifier string
-        sleep 15
-        while :
-        do
-            fullstatus=`xcrun altool --notarization-info "$uuid" ${NOTARY} 2>&1`  # get the status
-            status1=`echo "$fullstatus" | grep 'Status\:' | awk '{ print $2 }'`
-            if [[ $status1 = "success" ]]; then
-                xcrun stapler staple "${dmg_name}.dmg"   #  staple the ticket
-                xcrun stapler validate -v "${dmg_name}.dmg"
-                echo "dmg Notarization success"
-                rm *dmg.zip
-                break
-                elif [[ $status1 = "in" ]]; then
-                echo "dmg Notarization still in progress, sleeping for 15 seconds and trying again"
-                sleep 15
-            else
-                echo "dmg Notarization failed fullstatus below"
-                echo "$fullstatus"
-                exit 1
-            fi
-        done
+        sudo xcrun notarytool submit "${dmg_name}.dmg.zip" ${NOTARY} --wait
     fi
     
     # Zip disk image for redistribution
     msg "Zipping disk image for redistribution:"
     mkdir "${PROJECT_NAME}_macOS_${MINIMUM_SYSTEM_VERSION}_${arch}_${PROJECT_FULL_VERSION}_folder"
-        ditto {"${PROJECT_NAME}_macOS_${MINIMUM_SYSTEM_VERSION}_${arch}_${PROJECT_FULL_VERSION}.dmg","rawtherapee-cli","${PROJECT_SOURCE_DATA_DIR}/INSTALL.txt"} "${PROJECT_NAME}_macOS_${MINIMUM_SYSTEM_VERSION}_${arch}_${PROJECT_FULL_VERSION}_folder"
+    cp {"${PROJECT_NAME}_macOS_${MINIMUM_SYSTEM_VERSION}_${arch}_${PROJECT_FULL_VERSION}.dmg","${PROJECT_NAME}.app/Contents/MacOS/rawtherapee-cli","${PROJECT_SOURCE_DATA_DIR}/INSTALL.readme.rtf"} "${PROJECT_NAME}_macOS_${MINIMUM_SYSTEM_VERSION}_${arch}_${PROJECT_FULL_VERSION}_folder"
+    mv "${PROJECT_NAME}_macOS_${MINIMUM_SYSTEM_VERSION}_${arch}_${PROJECT_FULL_VERSION}_folder/INSTALL.readme.rtf" "${PROJECT_NAME}_macOS_${MINIMUM_SYSTEM_VERSION}_${arch}_${PROJECT_FULL_VERSION}_folder/install-readme.txt"
+    codesign -s "${CODESIGNID}" -i com.rawtherapee.rawtherapee-cli -f "${PROJECT_NAME}_macOS_${MINIMUM_SYSTEM_VERSION}_${arch}_${PROJECT_FULL_VERSION}_folder/rawtherapee-cli"
     zip -r "${PROJECT_NAME}_macOS_${MINIMUM_SYSTEM_VERSION}_${arch}_${PROJECT_FULL_VERSION}.zip" "${PROJECT_NAME}_macOS_${MINIMUM_SYSTEM_VERSION}_${arch}_${PROJECT_FULL_VERSION}_folder/"
     if [[ -n $NIGHTLY ]]; then
         cp "${PROJECT_NAME}_macOS_${MINIMUM_SYSTEM_VERSION}_${arch}_${PROJECT_FULL_VERSION}.zip" "${PROJECT_NAME}_macOS_${arch}_latest.zip"
     fi
+    if [[ -n $CONTINUOUS ]]; then
+        BRANCH=$(git branch --show-current)
+        if test -z "${BRANCH}"; then
+            BRANCH=$(git rev-parse --short HEAD)
+        fi
+        mv "${PROJECT_NAME}_macOS_${arch}_latest.zip" "${PROJECT_NAME}_${BRANCH}_macOS_${CMAKE_BUILD_TYPE}.zip"
+    fi
 }
+
 CreateDmg
 msg "Finishing build:"
+# Clean up items
+rm *app.zip
+rm *dmg.zip
+rm univ.zip
 echo "Script complete."
